#ifndef RENDERER_H
#define RENDERER_H

#include <stdint.h>

#include "SDL.h"
#include "shippingHandler.h"

#include "../interface/Button.h"
#include "../interface/HelpBox.h"
#include "../interface/ListBox.h"
#include "../interface/TextFeeder.h"
#include "../interface/PowerSelector.h"
#include "../interface/FormattedText.h"

#include "../util/ResourceHandler.h"

#include "../ruleset/Map.h"
#include "../ruleset/MapUnit.h"
#include "../ruleset/Player.h"
#include "../ruleset/Store.h"
#include "../ruleset/BattleAbility.h"

#include "GraphicTransformation.h"
#include "AudioTransformation.h"
#include "AnimatedState.h"

#include <stdint.h>

#include <vector>
#include <string>
using namespace std;

//the Renderer's job is to position and align everything, drawing it to the surface.
//GameState should pass whatever it needs to draw to Renderer. Renderer should load resources for these items.
//Renderer only has the information it needs to draw stuff. It needs to be told, for example, what the map looks like because it shouldn't store ingame information like that.

class Renderer {
 public:
  Renderer ();
  ~Renderer ();
  bool initialize ();

  //loading resources
  uint8_t loadResourceDebug (const string &);
  uint8_t clearResourceDebug (const string &);

  uint8_t loadResourceMap (Map & mep, MapUnit &, vector <MapUnit> &);
  uint8_t clearResourceMap ();

  uint8_t loadResourceMapEnemy ();
  uint8_t clearResourceMapEnemy();

  uint8_t loadResourcesHelp ();
  uint8_t clearResourcesHelp ();

  void loadResourceBattle (PowerSelector &);
  void clearResourceBattle (PowerSelector &);

  void setResourceEngine (shippingHandler *);
  shippingHandler * getResourceEngine ();

  /*
    each output screen is basically for a different game state. The
    Renderer knows nothing about this and it is up to GameState to pass
    the correct variables to it.

    These should be called by the overarching handling function.
  */
  void screenDelay ();

  //Startup: Render loads data to know what data to load and where to put it.
  void startupMainMenu ();
  void startupMapScreen (vector <MapUnit> & enemyMapUnits, MapUnit & playerMapUnit);
  void startupBattle (PowerSelector & ps, vector<TextFeeder> &);

  //Output: Draws and plays the state onscreen.
  void outputMainMenu (vector <string> & stringVect, vector <Button> & buttonVect, uint32_t);
  void outputBattleSwoosh (const string & title);
  void outputMapScreen (vector <string> &, vector <Button> &, Map &, MapUnit &, vector <MapUnit> &);
  void outputMapTerrain (Map & mep, MapUnit &, vector <MapUnit> &);
  void outputNewLevelAnnouncement ();
  void outputGameOver (vector <string> &, vector <Button> &, uint32_t);
  void outputCredits (vector <string> & sV, vector <Button> & bV);
  void outputPlayerPhase (vector <string> &, vector <Button> &, Player &, PowerSelector &, vector<BattleAbility> &, vector<TextFeeder> &);
  void outputEnemyPhase (vector <string> &, vector <Button> &, Player &);
  void outputRewards (vector <string> &, vector <Button> &, Player &, vector <uint8_t> &, uint32_t);
  void outputCheckForResult ();
  void outputInventory (vector <string> &, vector <Button> &, vector <ListBox> &, Player &, Store &, bool, bool);
  void outputAbility (vector <string> &, vector <Button> &, vector <ListBox> &, Player &, Store &, bool, bool);
  void outputGameMenu (vector <string> &, vector <Button> &);
  void outputContinue (vector <string> &, vector <Button> &);
  void outputChooseYourCharacter (vector <string> &, vector <Button> &);
  void outputChooseYourPowers (vector <string> &, vector <Button> &, Player &);
  void outputLevelComplete ();
  void outputNewGame (vector <string> & sV, vector <Button> & bV);
  void outputSave (vector <string> & sV, vector <Button> & bV);

  void positionHelpBoxText (HelpBox & newHB);
  void positionHelpBox (vector <HelpBox> & helpBoxVect, HelpBox & newHB);
  void outputHelpBoxStack (vector <HelpBox> & helpBoxVect);

  //Renderer reporting data (usually object placement)
  void getDebugMapScreenCoordinates (Map & mep, int32_t & mapLeftSide, int32_t & mapTopSide, int32_t & mapWidth, int32_t & mapHeight);


  //Helper function that sets up variables for shippingHandler::addFormattedTextToFixedDimensions against a textFeeder.
  void addFormattedTextToTextFeeder (const string & stringToAdd,
                                     TextFeeder & tf,
                                     bool forceNewLine=false,
                                     int colorIndex=0,
                                     int lineLimit=-1,
                                     const string & helpRef="NULL_INDEX");

 private:
  //sub states to display
  void outputDebugAbilitiesAvailable (Player &, vector <ListBox> &);
  void outputDebugAbilitiesEquipped (Player &, vector <ListBox> &);
  void outputDebugStoreAbilities (Player &, Store &, vector <ListBox> &);
  void outputDebugEquipment (Player &, vector <ListBox> &);
  void outputDebugStoreItems (Store &, vector <ListBox> &);
  void outputDebugPlayer1 (Player &);
  void outputDebugButtons (vector <string> &, vector <Button> &, uint8_t);
  void outputDebugLevelUp (Player &, vector <uint8_t> &);

  //printing objects
  void buttonDebugPlacement (vector <Button> &);
  void listBoxDebugPlacement (vector <ListBox> &);

  void outputListBox (ListBox & lb, vector <string> &, vector <bool> & vectBool);
  void outputTextFeeder (TextFeeder & tf);
  void outputPowerSelector (PowerSelector &, vector<BattleAbility> &);

  //using ResourceHandlers to automate resource handling.

  //load the correct type of resource and store the results into this resource handler.
  bool loadToResourceHandler(const string & key, const string & resourceName, int resourceType, ResourceHandler & rh);
  //remove the resource from this resource handler. Unload it, too.
  bool removeFromResourceHandler(const string & key, ResourceHandler & rh);

  //shippingHandler that handles the low-level drawing
  shippingHandler * sdlRE;

  string fontHeaderIndex, fontTextIndex;
  string buttonDebugDownIndex, buttonDebugUpIndex;
  string helpBoxBackgroundIndex;
  string listBoxBackground, listBoxUpArrow, listBoxDownArrow, listBoxScrollWindow, listBoxScrollBar, listBoxSelected;
  vector <string> mapGraphics;
  vector <string> mapUnitGraphics;

  string tempToon;
  AnimatedState tempAnimatedState;

  string testTextFeederBackground;
  TextFeeder testTextFeeder;

  string battleBackgroundGraphic;
  string battleClickzone;
  string battlePlayerInterfaceBackground;
  string battlePlayerToon;
  AnimatedState battlePlayerToonAnimatedState;

  ResourceHandler powerSelectorResources;
  ResourceHandler battleResources;
};

#endif
