#include "shippingHandler.h"
#include "../util/fileUtils.h"


void shippingHandler :: drawBitmap (const string & keyName, int32_t xPos, int32_t yPos, bool respectCamera, GraphicTransformation & gt, uint8_t orientation)
{
  //draws a source at the given coordinates.
  //respectCamera : If false, xPos and yPos are screen coordinates and ignore camera scroll.
  //orientation uses bit masks.
  //Horizontal
  //0x1 : Put text to the right of the coordinate.
  //0x2 : Put text to the left of the coordinate.
  //Otherwise, horizontally center the text.
  //Vertical
  //0x4 : Put text Below the coordinate.
  //0x8 : Put text Above the coordinate.
  //Otherwise, vertically center the text.

  if (keyName.empty() || keyName == "NULL_INDEX")
    return;

  map <string, GraphicBitmap> :: iterator iter2;
  iter2 = graphicBitmaps.find (keyName);

  if (iter2 == graphicBitmaps.end())
    return;

  SDL_Rect dest, transformOut;

  int destX = xPos, destY = yPos, destW, destH;
  int16_t anchorPointX, anchorPointY;
  float scaleX=1.0, scaleY=1.0, angle=0.0;
  SDL_Surface *tempSurface=NULL;
  GraphicTransformation gtCopy(gt);

  if (respectCamera)
    {
      destX = (screenResolutionX/2 + xPos - observerX);
      destY = (screenResolutionY/2 + yPos - observerY);
    }
  else
    {
      destX = xPos;
      destY = yPos;
    }

  destW = iter2->second.getPic()->w;
  destH = iter2->second.getPic()->h;

  if (orientation & 0x1)
    destX += destW/2;
  else if (orientation & 0x2)
    destX -= destW/2;
  else
    destX += 0;

  if (orientation & 0x4)
    destY += destH/2;
  else if (orientation & 0x8)
    destY -= destH/2;
  else
    destY += 0;

  dest.x = destX;
  dest.y = destY;
  dest.w = destW;
  dest.h = destH;

  //If you don't need to transform the image:
  if ((gt.getTransformFlag() & GRAPHIC_TRANSFORMATION_SCALE) == 0 &&
      (gt.getTransformFlag() & GRAPHIC_TRANSFORMATION_ROTATION) == 0)
    {
      //set the endpoints flag and add endpoints to move the center point off of the center of the image.
      dest.x -= destW/2;
      dest.y -= destH/2;

      SDL_BlitSurface(iter2->second.getPic(), NULL, screenBuffer, &dest);
    }
  else
    {
      //else, use sge's transformation.
      //you need to calculate the new x & y coordinates based on the rotated width & height.

      //Whatever you give as the anchor point, it will rotate and output around that.

      //SDL_Rect sge_transform(SDL_Surface *src, SDL_Surface *dst, float angle, float xscale, float yscale, Uint16 px, Uint16 py, Uint16 qx, Uint16 qy, Uint8 flags)
      //set the endpoints flag and add endpoints to move the center point off of the center of the image.
      anchorPointX=destW/2;
      anchorPointY=destH/2;

      if ((gt.getTransformFlag () & GRAPHIC_TRANSFORMATION_ROTATION) > 0)
        {
          //scale 0-256 to 0-360.
          angle=byteDegreesToDegrees(gt.getRotationAngle());
        }

      if (gt.getTransformFlag()|GRAPHIC_TRANSFORMATION_SCALE)
        {
          scaleX=gt.getScaleWidth()/16.0;
          scaleY=gt.getScaleHeight()/16.0;
        }
      else
        {
          scaleX=1.0;
          scaleY=1.0;
        }

      tempSurface = sge_transform_surface(iter2->second.getPic(),
                                          SDL_MapRGBA(iter2->second.getPic()->format, 0,0,0,0),
                                          angle, scaleX, scaleY, 0);

      if (gt.getTransformFlag()|GRAPHIC_TRANSFORMATION_ROTATION)
        {
          gtCopy.setTransformFlag(GRAPHIC_TRANSFORMATION_ROTATION);
          gtCopy.setRotationAngle(gt.getRotationAngle());
          gtCopy.setRotationAnchor(gt.getRotationAnchorX(), gt.getRotationAnchorY());
          anchorPointX=0;
          anchorPointY=0;
          transformPoint(anchorPointX, anchorPointY, gtCopy);
        }
      else
        {
          anchorPointX=0;anchorPointY=0;
        }

      transformOut.x=destX-(tempSurface->w/2)+anchorPointX;
      transformOut.y=destY-(tempSurface->h/2)+anchorPointY;
      transformOut.w=tempSurface->w;
      transformOut.h=tempSurface->h;

      SDL_BlitSurface(tempSurface, NULL, screenBuffer, &transformOut);

      SDL_FreeSurface(tempSurface);
      tempSurface=NULL;
    }
}


bool shippingHandler :: loadBitmap (string & keyName)
{
  //simply a front to loadResource.
  if (keyName.empty() || keyName == "NULL_INDEX")
    {
      keyName="NULL_INDEX";
      return false;
    }

  return loadResource (keyName, "bitmap");
}

bool shippingHandler :: loadBitmapHelper (const string & index, GraphicBitmap & fram)
{
  //loads the pic if it hasn't been already loaded. Otherwise skip.
  //returns true if it loads a pic.
  SDL_Surface * temp = NULL;

  //parse the xml file and find the filename for the given index
  xmlDocPtr doc;
  xmlXPathContextPtr xpathCtx;
  string filename;

  if (!prepareXMLFile (doc, xpathCtx, "resources/BitmapManifest.xml", ""))
    {
      return false;
    }

  xmlChar *keyword;
  keyword = getFirstElementUsingXPath (doc, xpathCtx, "/BitmapManifest/Bitmap[@ID='", index, "']/Filename");

  if (keyword!=NULL)
    filename = (const char *)keyword;
  else
    {
      xmlXPathFreeContext(xpathCtx);
      xmlFreeDoc(doc);
      return false;
    }

  xmlXPathFreeContext(xpathCtx);
  xmlFreeDoc(doc);
  xmlFree(keyword);

  temp = IMG_Load(filename.c_str());

  if (temp == NULL){
    writeToErrorLog ("loadBitmapHelper: Image failed to load.", 2);
    writeToErrorLog (index.c_str(), 2);
    return false;
  }

  //this converts an SDL_Surface to match the screen's stats.
  fram.setPic(SDL_DisplayFormatAlpha (temp));

  SDL_FreeSurface(temp); //Now release it, we're done.
  return true;
}

void shippingHandler :: releaseBitmap (const string & keyName)
{
  if (keyName.empty() || keyName == "NULL_INDEX")
    return;

  map <string, GraphicBitmap> :: iterator iter2;
  iter2 = graphicBitmaps.find (keyName);

  if (iter2 == graphicBitmaps.end())
    return;

  iter2->second.decRefCount();

  if (iter2->second.getRefCount()==0)
    {
      SDL_FreeSurface(iter2->second.getPic());
      iter2->second.setPic(NULL);
      graphicBitmaps.erase(iter2);
    }
}
