#include "shippingHandler.h"

//Dimensions are fixed. Text is added so it wraps within the given width.
void shippingHandler::addFormattedTextToFixedDimensions (const string & stringToAdd,
                                                         const string & fontName,
                                                         uint16_t heightFactor,
                                                         vector <FormattedText> & fmt,
                                                         uint16_t containerWidth,
                                                         bool forceNewLine,
                                                         int colorIndex,
                                                         int lineLimit,
                                                         const string & helpRef)
{
  int xPos=0, yPos=0;
  uint16_t leftoverWidth;
  bool done=false, wrappedNewLine;
  string stringToUse = stringToAdd, stringLeftover="";
  uint16_t textWidth, textHeight;
  FormattedText f;
  leftoverWidth=containerWidth-getWidthOfLastLine(fmt);
  int response;
  int numLinesAdded=0;

  if (stringToAdd.empty())
    stringToUse="A";

  while (!done)
    {
      wrappedNewLine=false;
      f.clear();
      //get the largest piece of text that will fit within the given width.
      response=getLargestStringToFitInWidth(stringToUse, stringLeftover, leftoverWidth, fontName);

      //if nothing will fit within the given space, set a newline, reset the width and get the largest piece of text that fits within the full width.
      if (response!=0)
        {
          wrappedNewLine=true;
          leftoverWidth=containerWidth;
          if (response==2)
            {
              //there isn't enough room to fit another letter on. The previous FormattedText should have a newline set.
              fmt.back().setNewline(true);
              getLargestStringToFitInWidth(stringToUse, stringLeftover, leftoverWidth, fontName);
            }
        }
      getTextDimensions(stringToUse, fontName, textWidth, textHeight);

      //get coordinates
      getNextFormattedTextCoordinate(fmt, xPos, yPos);

      //apply heightFactor. It's out of 16ths.
      textHeight *= heightFactor;
      textHeight /= 16;

      //Now set up a new FormattedText.
      if (stringToAdd.empty())
        f.setText("");
      else
        f.setText(stringToUse);
      f.setColorIndex(colorIndex);
      f.setHelpIndex(helpRef);
      f.setPosition(xPos, yPos);
      f.setDimensions(textWidth,textHeight);

      if (!wrappedNewLine)
        f.setNewline(forceNewLine);
      else
        {
          numLinesAdded++;
          f.setNewline(true);
        }

      //finally push the FormattedText to the stack.
      fmt.push_back(f);

      //Update stringToUse. if the text is blank, bail
      stringToUse=stringLeftover;
      if (stringToUse.empty())
        done=true;

      //check the number of lines against the limit.
      if (lineLimit > -1 && numLinesAdded >= lineLimit)
        done=true;
    }
}

//get the length of the last line.
uint16_t shippingHandler::getWidthOfLastLine(vector<FormattedText>& fmt)
{
  vector <FormattedText> :: iterator fmtIter = fmt.begin(), fmtStartOfLastLine = fmt.begin(), fmtNextLine, fmtInLine;

  if (fmtIter==fmt.end())
    return 0;

  bool newLineWarning=false;
  bool done=false;
  uint16_t ans=0;
  //get the start of the last line.
  while (fmtIter!=fmt.end())
    {
      newLineWarning=false;
      fmtStartOfLastLine=fmtIter;
      //jump to the end of each line.
      done=false;
      while (!done)
        {
          if (fmtIter==fmt.end()||fmtIter->getNewline())
            {
              done=true;
              if (fmtIter->getNewline())
                newLineWarning=true;
            }
          else
            fmtIter++;
        }

      if (fmtIter!=fmt.end())
        fmtIter++;
    }

  //exception: If the last line ends with a newline, ans = 0
  if (newLineWarning)
    return 0;

  //count up the width of each FormattedText.
  fmtIter=fmtStartOfLastLine;
  ans=0;
  while (fmtIter!=fmt.end())
    {
      ans+=fmtIter->getWidth();
      fmtIter++;
    }
  return ans;
}

//getting the biggest string that will fit.
//return 0 if string completely fits.
//return 1 if some of string can fit on the line.
//return 2 if none of the string can fit on the line.
int shippingHandler::getLargestStringToFitInWidth(string & stringToUse, string & stringLeftover, uint16_t leftoverWidth, const string & fontName)
{
  if (stringToUse.empty())
    {
      stringLeftover="";
      return 0;
    }

  //1) If the first character doesn't fit, return false.
  string testString=stringToUse.substr(0,1);
  uint16_t stringWidth, stringHeight;

  getTextDimensions(testString, fontName, stringWidth, stringHeight);

  if (stringWidth>leftoverWidth)
    {
      stringLeftover="";
      return 2;
    }

  //2) If the entire string fits, use the whole string, no leftover, return true.
  getTextDimensions(stringToUse, fontName, stringWidth, stringHeight);

  if (stringWidth<=leftoverWidth)
    {
      stringLeftover="";
      return 0;
    }

  //3) Otherwise, do a binary search to find the largest string that fits.
  //   a) Low bound = 0, Up bound = length of string. First guess = halfway.
  int lowerBound=0, upperBound=stringToUse.size(), guess=(lowerBound+upperBound)/2, precision=1;

  //   b) until lowbound +1 = upbound
  while (upperBound-lowerBound>precision)
    {
      //1) get length of substring
      testString=stringToUse.substr(0,guess);
      getTextDimensions(testString, fontName, stringWidth, stringHeight);
      //2) if it's exact, set up and low bound to guess.
      if (stringWidth==leftoverWidth)
        {
          lowerBound=guess;
          upperBound=guess;
        }
      //3) if it's too long, upbound = guess
      else if (stringWidth > leftoverWidth)
        upperBound=guess;
      //4) if it's too short, lowbound = guess
      else if (stringWidth < leftoverWidth)
        lowerBound=guess;

      guess=(lowerBound+upperBound)/2;
    }

  //   c) lowbound is the length of the string to use.
  //      stringToUse = [0,lowbound)
  //      stringLeftover = [lowbound, length)
  stringLeftover=stringToUse.substr(lowerBound);
  stringToUse=stringToUse.substr(0,lowerBound);
  return 1;
}

//returns the coordinates of the next formatted text to be added to the given vector.
void shippingHandler::getNextFormattedTextCoordinate(vector<FormattedText> & fmt, int & xPos, int & yPos)
{
  vector <FormattedText> :: iterator fmtIter=fmt.begin();
  xPos=0;
  yPos=0;

  int maxHeight=0;

  while (fmtIter!=fmt.end())
    {
      xPos+=fmtIter->getWidth();
      if (fmtIter->getHeight()>maxHeight)
        maxHeight=fmtIter->getHeight();
      if (fmtIter->getNewline())
        {
          xPos=0;
          yPos+=maxHeight;
          maxHeight=0;
        }
      fmtIter++;
    }
}

//arranges the position of the given text.
void shippingHandler::arrangeFormattedText (vector <FormattedText> :: iterator & fmtIter,
                                            vector <FormattedText> :: iterator & fmtIterEnd)
{
  int xPos, yPos;
  xPos=0; yPos=0;
  int maxHeight=0;

  while (fmtIter!=fmtIterEnd)
    {
      fmtIter->setPosition(xPos, yPos);
      xPos+=fmtIter->getWidth();
      if (fmtIter->getHeight()>maxHeight)
        maxHeight=fmtIter->getHeight();
      if (fmtIter->getNewline())
        {
          xPos=0;
          yPos+=maxHeight;
          maxHeight=0;
        }
      fmtIter++;
    }
}

//Formatted text and its newlines are fixed. Calculate the minimum width and height needed to contain it.
void shippingHandler::getContainerDimensionWithFixedText (uint16_t & containerWidth, uint16_t & containerHeight,
                                         vector <FormattedText> :: iterator & fmtIterStart,
                                         vector <FormattedText> :: iterator & fmtIterEnd,
                                         const string & fonttype)
{
}
