#include "SDLResourceAnimation.h"

#include "SDL.h"
#include "SDL_mixer.h"

#include "SDLResourceEngine.h"
#include "SDLResourceSoundSource.h"
#include "SDLResourceFrameSource.h"
#include "../fileUtils.h"
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#include <vector>
using namespace std;

SDLResourceAnimation :: SDLResourceAnimation (SDLResourceEngine * sre)
{
  frames = new vector<string> (0);
  xAnchor = new vector<int32_t> (0);
  yAnchor = new vector<int32_t> (0);
  timeForFrame = new vector<uint16_t> (0);
  soundAtStartUp = "NULL_INDEX"; //goes when play () is called
  soundAtLoop = "NULL_INDEX"; //goes it loops
  soundAtStop = "NULL_INDEX"; //goes when out of loops, or explicitly stopped with stop ()
  frameToRestartAt = 0;
  numLoops = -1;

  sdlRE = sre;
}

void SDLResourceAnimation :: releaseFrameSources ()
{
  //this releases the loaded frame sources.
  if (frames == NULL)
    return;

  vector <string> :: iterator the_iterator;
  the_iterator = frames -> begin ();

  while(the_iterator != frames -> end ()) {
    sdlRE -> releaseFrameSource ((*the_iterator));
    the_iterator ++;
  }

  //delete the frames, they are strings now!
}


void SDLResourceAnimation :: releaseSoundSources ()
{
  //this releases the loaded frame sources.
  sdlRE -> releaseSoundSource (soundAtStartUp);
  sdlRE -> releaseSoundSource (soundAtLoop);
  sdlRE -> releaseSoundSource (soundAtStop);
}

SDLResourceAnimation :: ~SDLResourceAnimation ()
{
  //CALL release* functions before you call the destructor! Otherwise you'll have a memory leak.

  releaseFrameSources ();
  releaseSoundSources ();

  //You cannot overload a destructor. The best I can do is write "Not closed correctly" into the error log.

  if (frames != NULL)
    delete frames;

  if (xAnchor != NULL)
    delete xAnchor;

  if (yAnchor != NULL)
    delete yAnchor;

  if (timeForFrame != NULL)
    delete timeForFrame;
}

uint8_t SDLResourceAnimation :: loadAnim (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  //This will go through the given XML document, get frame and sound information.
  //It will ask the Resource Engine to load frames and sounds then stores them into the array.

  uint16_t loader;
  string loaderString;
  xmlChar * element = NULL;
  
  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/SoundIndexAtStart", "");
  sdlRE -> loadSound ((const char *)element, soundAtStartUp);
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/SoundIndexAtLoop", "");
  sdlRE -> loadSound ((const char *)element, soundAtLoop);
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/SoundIndexAtEnd", "");
  sdlRE -> loadSound ((const char *)element, soundAtStop);
  xmlFree (element);
  
  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Loops", "");
  if (element == NULL)
    numLoops = 0;
  else if (xmlStrcasecmp (element, (const xmlChar *)"infinite") == 0)
    numLoops = -1;
  else
    numLoops = int(xmlXPathCastStringToNumber(element));
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/RestartFrame", "");
  if (element == NULL)
    frameToRestartAt = 0;
  else
    frameToRestartAt = int(xmlXPathCastStringToNumber(element));
  xmlFree (element);

  loader = 0;
  
  //load frames
  string xpathFrame = xpath + "/Frame";

  string idString;
  char searchString [255];

  xmlXPathObjectPtr xpathObj;
  xpathObj=getElementsUsingXPath (doc, xpathCtx, xpathFrame, "", "");

  int numFrames = 0;

  //No frames present.
  if (xpathObj!=NULL)
    numFrames = xpathObj->nodesetval->nodeNr;
  else
    return false;
  
  xmlXPathFreeObject(xpathObj);
  
  for (int i=0; i<numFrames; i++)
    {
      sprintf (searchString, "[%u]", i+1);
      idString=xpathFrame+searchString;
      if (loadFrame (doc, xpathCtx, idString))
	loader++;
    }

  if (loader <= 0)
    return false; //at least 1 frame must be loaded.

  return true;
}

uint8_t SDLResourceAnimation :: loadFrame (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  //reads information from the given XML file.
  uint16_t loader;
  string loaderString, loaderString2;
  xmlChar * element, *secondElement, *thirdElement;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/FrameManifestIndex", "");
  loaderString.assign (((const char *)element));
  sdlRE -> loadFrame (loaderString, loaderString2);
  if (loaderString2 != "NULL_INDEX")
    frames -> push_back (*(new string (loaderString2)));
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Duration", "");
  if (element == NULL)
    loader = 33;
  else
    loader = int(xmlXPathCastStringToNumber(element));
  timeForFrame -> push_back (loader);
  xmlFree (element);

  int temp;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Anchor", "");
  if (element == NULL)
    {
      xAnchor -> push_back (0);
      yAnchor -> push_back (0);
    }
  else
    {

      if (xmlStrchr (element, (const xmlChar)' ') == NULL)
	thirdElement = xmlStrndup (element, xmlStrchr(element, (const xmlChar)',')-element);
      else
	thirdElement = xmlStrndup (element, xmlStrchr(element, (const xmlChar)' ')-element);

      temp = int(xmlXPathCastStringToNumber(thirdElement));
      xAnchor -> push_back ((int32_t)(temp));

      //find the first space
      if (xmlStrchr (element, (const xmlChar)' ') == NULL && xmlStrchr (element, (const xmlChar)',') == NULL)
	yAnchor -> push_back (0);
      else
	{
	  if (xmlStrchr (element, (const xmlChar)' ') == NULL)
	    secondElement = xmlStrdup (xmlStrchr(element, (const xmlChar)','));
	  else
	    secondElement = xmlStrdup (xmlStrchr(element, (const xmlChar)' '));
      
	  temp = int(xmlXPathCastStringToNumber(secondElement+1)); //skip the first character, usualy a + or ,
	  yAnchor -> push_back ((int32_t)(temp));
	  xmlFree (secondElement);
	}
    }
  xmlFree (element);
  return true;
}

const string & SDLResourceAnimation :: getSoundAtStartup ()
{
  return soundAtStartUp;
}

const string & SDLResourceAnimation :: getSoundAtLoop ()
{
  return soundAtLoop;
}

const string & SDLResourceAnimation :: getSoundAtStop ()
{
  return soundAtStop;
}

const string & SDLResourceAnimation :: getFrameIndex (uint16_t frame)
{
  if (frame >= frames -> size ())
    return *(new string ("NULL_INDEX"));

  return frames -> at (frame);
}

int32_t SDLResourceAnimation :: getXAnchor (uint16_t frame)
{
  if (frame >= xAnchor -> size ())
    return 0;

  return xAnchor -> at (frame);
}

int32_t SDLResourceAnimation :: getYAnchor (uint16_t frame)
{
  if (frame >= yAnchor -> size ())
    return 0;

  return yAnchor -> at (frame);
}

uint16_t SDLResourceAnimation :: getTimeForFrame (uint16_t frame)
{
  if (frame >= timeForFrame -> size ())
    return 0;

  return timeForFrame -> at (frame);
}

uint8_t SDLResourceAnimation :: getRestartFrame ()
{
  return frameToRestartAt;
}

uint16_t SDLResourceAnimation :: getNumLoops ()
{
  return numLoops;
}

uint8_t SDLResourceAnimation :: getNumFrames ()
{
  return frames -> size ();
}
