#include "SDL.h"
#include "SDLResourceEngine.h"
#include "SDLResourceFrameBuffer.h"
#include "SDLResourceFrameSource.h"
#include "SDLResourceGFX.h"
#include "SDLResourceMusic.h"
#include "SDLResourceSoundBuffer.h"
#include "SDLResourceSoundSource.h"
#include "SDLResourceToon.h"
#include "SDLResourceFont.h"
#include "../fileUtils.h"

#include <string>
#include <ctime>
#include <iostream>
using namespace std;

SDLResourceEngine :: SDLResourceEngine()
{
  screen = NULL;

  frameBuffers.clear();
  frameSources.clear();

  soundBuffers.clear();
  soundSources.clear();

  gfxSources.clear();
  cgSources.clear();

  musicResource = new SDLResourceMusic;
  musicIndex = "NULL_INDEX";

  fonts.clear();

  toons = new vector <SDLResourceToon *> (0);

  observerX = 0;
  observerY = 0;

  minScreenResolutionX = 640;
  minScreenResolutionY = 480;
  windowedScreen = true;

  screenResolutionX = 1024;
  screenResolutionY = 768;

  bkgColorR = 0;
  bkgColorB = 0;
  bkgColorG = 0;

  loadCounter = 0;
}

SDLResourceEngine :: ~SDLResourceEngine()
{
  vector <SDLResourceToon *> :: iterator iter5;
  iter5 = toons -> begin ();

  while(iter5 != toons -> end ()) {
    delete (*iter5);
    iter5 ++;
  }

  //remove all frameSources
  frameSources.clear ();

  //delete everything in the vector
  frameBuffers.clear ();

  //clearing gfx
  map <string, SDLResourceGFX *> :: iterator gfxIterator;
  gfxIterator = gfxSources.begin ();
  while(gfxIterator != gfxSources.end ()) {
    delete (*gfxIterator).second;
    gfxIterator ++;
  }

  //remove all soundSources
  map <string, SDLResourceSoundSource *> :: iterator ssiter;
  ssiter = soundSources.begin ();

  while(ssiter != soundSources.end ()) {
    (*ssiter).second -> releaseSoundBuffer ();
    delete (*ssiter).second;
    ssiter ++;
  }

  vector <SDLResourceSoundBuffer *> :: iterator sbiter;
  sbiter = soundBuffers.begin ();

  while(sbiter != soundBuffers.end ()) {
    delete ((*sbiter));
    sbiter ++;
  }

  //fonts
  map <string, SDLResourceFont*> :: iterator iter6;
  iter6 = fonts.begin ();

  while(iter6 != fonts.end ()) {
    delete (*iter6).second;
    iter6 ++;
  }

  //cgSources
  map <string, SDLResourceComplexGFX *> :: iterator iter7;
  iter7 = cgSources.begin ();

  while(iter7 != cgSources.end ()) {
    delete (*iter7).second;
    iter7 ++;
  }

  if (musicResource != NULL)
    delete musicResource;

  if (toons != NULL)
    delete toons;
  
  TTF_Quit ();
  //SDL_Mixer
  Mix_CloseAudio();
  //shtus down SDL processes.  
  SDL_FreeSurface(screen);
  //when quitting
  SDL_Quit ();
}

int8_t SDLResourceEngine :: SDLStartup ()
{
  //this starts SDL.
  //returns true on success
  SDL_ClearError ();
  //first initalize SDL
  if (SDL_Init (SDL_INIT_VIDEO | SDL_INIT_AUDIO) != 0){
    writeToErrorLog ("SDLStartup failed", 0);
    writeToErrorLog (SDL_GetError (), 1);
    return false;
  }

  SDL_Rect ** videoModes;

  screenResolutionX = minScreenResolutionX;
  screenResolutionY = minScreenResolutionY;

  SDL_ClearError ();

  if (windowedScreen)
    screen = SDL_SetVideoMode (screenResolutionX, screenResolutionY, 32, SDL_DOUBLEBUF);
  else
    screen = SDL_SetVideoMode (screenResolutionX, screenResolutionY, 32, SDL_DOUBLEBUF | SDL_FULLSCREEN);

  if (screen == NULL)
    {
      //get the next available resolution
      if (windowedScreen)
	videoModes=SDL_ListModes(NULL, SDL_DOUBLEBUF);
      else
	videoModes=SDL_ListModes(NULL, SDL_DOUBLEBUF|SDL_FULLSCREEN);

      if (videoModes == NULL)
	{
	  writeToErrorLog ("Set video mode failed. No available resolutions with these properties: 32 bit, doublebuffered", 0);
	  writeToErrorLog (SDL_GetError (), 1);
	  return false;
	}
      else //if it videoModes == -1, then the resolution should have set before and shouldn't have been here.
	{
	  //videoModes is in no particular order. It should be filtered for the next smallest size.
	  //pick the smallest legal size that is above the minimum resolution.
	}
    }

  /*
  if (videoModes == NULL)
    cout << "Returned NULL" << endl;
  else if (videoModes == (SDL_Rect **)-1)
    cout << "Returned -1" << endl;
  else
    for(int i=0;videoModes[i];++i)
      cout << videoModes[i]->w << " x " << videoModes[i]->h << endl;
  */

  if (screen == NULL){
    writeToErrorLog ("Set video mode failed.", 0);
    writeToErrorLog (SDL_GetError (), 1);
    return false;
  }
  SDL_ClearError ();
  if (Mix_OpenAudio (MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 2, 2048) < 0)
    {
      writeToErrorLog ("Error in Mix_OpenAudio", 0);
      writeToErrorLog (Mix_GetError (), 1);
      return false;
    }
  
  //you need to open the sound system before you query its stats. This is important if this computer doesn't match default settings.
  if (Mix_QuerySpec (&audioFreq, &audioFormat, &audioChannels) == 0)
    {
      writeToErrorLog ("Error while querying sound system", 0);
      return false;
    }
  /*
  Mix_CloseAudio(); //<--this is choking gdb
  SDL_ClearError ();
  //reopen with the system's settings so the sound clips convert correctly
  if (Mix_OpenAudio (audioFreq, audioFormat, audioChannels, 2048) < 0)
    {
      writeToErrorLog ("Error in Mix_OpenAudio", 0);
      writeToErrorLog (Mix_GetError (), 1);
      return false;
    }
  */
  SDL_ClearError ();
  if(TTF_Init()==-1) {
    writeToErrorLog("Error in TTF", 0);
    writeToErrorLog(TTF_GetError(), 1);
    return false;
  }

  return true;
}

void SDLResourceEngine :: loadFrame (const string & frameName, string & keyName)
//loads a frame into a buffer, adds a source, assigns a source to the buffer, then returns a reference to the new source.
{
  //keyName is set to the reference if it loads.
  //keyName is set to NULL_INDEX if there is an error.

  if (frameName.empty() || frameName == "NULL_INDEX")
    {
      keyName = "NULL_INDEX";
      return;
    }
  if (frameSources.size() >= SDL_RESOURCE_FRAME_SOURCE_MAX)
    {
      writeToErrorLog ("loadFrame: Maximum number of Frame Sources reached", 1);
      keyName = "NULL_INDEX";
      return;
    }

  //When loading, we try to recycle buffers.
  //There are 3 outcomes:
  //1) There is a buffer that already contains what we're looking for. Make a new source, point it at this buffer and we're done.
  //2) No buffer has what we want, but there is an empty buffer lying around. Load the info into that, make a new source and load it into there.
  //3) No buffer has what we want, and there are no extra buffers lying around. Make a new buffer, load the info into that and make a new source and load it into there.

  uint8_t outcome = 3;
  SDLResourceFrameSource sourceToUse;

  vector <SDLResourceFrameBuffer> :: iterator iter2;
  vector <SDLResourceFrameBuffer> :: iterator bufferHasItAlready;
  vector <SDLResourceFrameBuffer> :: iterator emptyBuffer;
  iter2=frameBuffers.begin();

  //See if there is a buffer that has this picture already.
  while(iter2 != frameBuffers.end () && (outcome != 1) ) {
    if (iter2 -> getPicIndex() == frameName)
      {
	bufferHasItAlready = iter2;
	outcome = 1;
      }
    
    if (iter2->getPicIndex().empty() || iter2->getPicIndex ()=="NULL_INDEX")
      {
	emptyBuffer = iter2;
	outcome = 2;
      }
    iter2 ++;
  }

  char indexStr [20];
  sprintf (indexStr, " %08x", loadCounter);
  keyName.assign (frameName);
  keyName.append (indexStr);

  if (outcome == 1)
    {
      //buffer is ready to go.
      sourceToUse.recieveFrameBuffer (&(*bufferHasItAlready));
      frameSources[keyName] = sourceToUse;
      loadCounter++; //increment load counter here.
      return;
    }

  if (outcome == 2)
    {
      //reassign buffer here
      if (emptyBuffer->loadPic (frameName) == false)
	{
	  writeToErrorLog ("loadFrame: frame not loaded", 1);
	  writeToErrorLog (frameName.c_str(), 2);
	  emptyBuffer->releasePic ();
	  keyName = "NULL_INDEX";
	  return;
	}
      sourceToUse.recieveFrameBuffer (&(*emptyBuffer));
      frameSources[keyName] = sourceToUse;
      loadCounter++; //increment load counter here.
      return;
    }

  SDLResourceFrameBuffer bufferToUse;

  if (outcome == 3)
    {
      //make a new buffer, load it here.
      //reassign buffer here
      if (bufferToUse.loadPic (frameName) == false)
	{
	  writeToErrorLog ("loadFrame: frame not loaded", 1);
	  writeToErrorLog (frameName.c_str(), 2);
	  bufferToUse.releasePic ();
	  keyName = "NULL_INDEX";
	  return;
	}
      frameBuffers.push_back(bufferToUse);
      sourceToUse.recieveFrameBuffer (&(bufferToUse));
      frameSources[keyName] = sourceToUse;
      loadCounter++; //increment load counter here.
      return;
    }

  return;
}

void SDLResourceEngine :: loadGFX (const string & gfxName, string & keyName)
//makes a new GFX resource and loads stats into it.
{
  //returns 0 if there is an error, otherwise returns index.
  if (gfxName.empty() || gfxName == "NULL_INDEX") //0 is a null
    {
      keyName = "NULL_INDEX";
      return;
    }

  if (gfxSources.size () >= SDL_RESOURCE_GFX_MAX)
    {
      writeToErrorLog ("loadGFX: Maximum number of GFX reached", 1);
      keyName = "NULL_INDEX";
      return;
    }

  SDLResourceGFX * bufferToUse = NULL;
  bool loadBuffer = true;

  //if something already has this, return that.
  map <string, SDLResourceGFX *> :: iterator iter2;
  iter2 = gfxSources.find (gfxName);

  if (iter2 != gfxSources.end())
    {
      bufferToUse = (*iter2).second;
      loadBuffer = false;
    }

  if (loadBuffer)
    {
      bufferToUse = new SDLResourceGFX ();

      if (bufferToUse -> loadGFX (gfxName) == false)
	{
	  writeToErrorLog ("loadGFX: GFX not loaded", 1);
	  delete bufferToUse;
	  keyName = "NULL_INDEX";
	  return;
	}
      gfxSources[gfxName]=bufferToUse;
    }
  keyName=gfxName;
  return;
}

void SDLResourceEngine :: loadSound (const string & soundName, string & keyName)
//loads a frame into a buffer, adds a source, assigns a source to the buffer, then returns a reference to the new source.
{
  //returns true if loaded.
  //returns false if there is an error or soundName is empty.

  if (soundName.empty() || soundName == "NULL_INDEX") //0 is a null
    {
      keyName = "NULL_INDEX";
      return;
    }

  if (soundSources.size() >= SDL_RESOURCE_SOUND_SOURCE_MAX)
    {
      writeToErrorLog ("loadSound: Maximum number of Sound Sources reached", 1);
      writeToErrorLog (soundName.c_str(), 2);
      keyName = "NULL_INDEX";
      return;
    }

  SDLResourceSoundSource * sourceToUse = NULL;
  SDLResourceSoundBuffer * bufferToUse = NULL;
  bool loadBuffer = true;

  //look for a buffer that has this frame already.
  vector <SDLResourceSoundBuffer *> :: iterator iter2;
  iter2 = soundBuffers.begin ();

  //See if there is a buffer that has this sound already.
  while(iter2 != soundBuffers.end () && bufferToUse == NULL) {
    if ((*iter2) -> getSoundIndex().empty() || (*iter2) -> getSoundIndex () == soundName || (*iter2) -> getSoundIndex () == "NULL_INDEX")
      {
        bufferToUse = *iter2;
        if ((*iter2)->getSoundIndex()==soundName)
          loadBuffer=false;
      }
    iter2 ++;
  }

  char indexStr [20];
  sprintf (indexStr, " %08x", loadCounter);
  keyName.assign (soundName);
  keyName.append (indexStr);

  //load if you need to.
  if (loadBuffer)
    {
      bufferToUse = new SDLResourceSoundBuffer ();
      if (bufferToUse -> loadSound (soundName) == false)
	{
	  delete bufferToUse;
	  writeToErrorLog ("loadSound: frame not loaded", 1);
	  writeToErrorLog (soundName.c_str(), 2);
	  bufferToUse -> releaseSound ();
	  keyName = "NULL_INDEX";
	  return;
	}
      soundBuffers.push_back(bufferToUse);
    }

  //OK, now there is a buffer with the info we want. Now we need to get a source ready.
  sourceToUse = new SDLResourceSoundSource ();

  soundSources[keyName]=sourceToUse;

  //OK, now assign the buffer to the source.
  sourceToUse -> recieveSoundBuffer (bufferToUse); //puts the frame into a source

  loadCounter++; //increment load counter here.

  //the name is the hash.
  return;
}

uint16_t SDLResourceEngine :: loadToon (const string & toonIndex)
//makes a new Toon resource and loads stats into it.
{
  //returns 0 if there is an error, otherwise returns index.
  if (toonIndex.empty () || toonIndex == "NULL_INDEX")
    return true;

  //Step 0: Find the first free source.
  if (toons -> size () >= SDL_RESOURCE_TOON_MAX)
    {
      writeToErrorLog ("loadToon: Maximum number of Toons reached", 1);
      return NULL_INDEX;
    }

  SDLResourceToon * bufferToUse = NULL;

  toons -> push_back (new SDLResourceToon (this));
  bufferToUse = toons -> back ();
  if (bufferToUse -> loadToon (toonIndex) == false)
    {
      writeToErrorLog ("loadToon: Toon not loaded", 1);
      delete bufferToUse;
      toons -> pop_back ();
      return NULL_INDEX;
    }

  //need to return its ID!
  return toons -> size ();
}

void SDLResourceEngine :: playSound (const string & sourceName, uint8_t volume, uint8_t leftPan)
{
  if (sourceName.empty() || sourceName == "NULL_INDEX")
    return;

  map <string, SDLResourceSoundSource *> :: iterator iter2;
  iter2 = soundSources.find (sourceName);

  if (iter2 == soundSources.end())
    return;

  SDLResourceSoundSource * sourceToUse = (*iter2).second;

  //play sound here

  //check to see if the sound is actually played.
  sourceToUse -> play (volume); //panning?
}

void SDLResourceEngine :: drawFrame (const string & sourceName, int32_t xPos, int32_t yPos, bool respectCamera, uint8_t orientation)
{
  //draws a source at the given coordinates.

  //respectCamera : If false, xPos and yPos are screen coordinates and ignore camera scroll.

  //orientation uses bit masks.
  //Horizontal
  //0x1 : Put text to the right of the coordinate.
  //0x2 : Put text to the left of the coordinate.
  //Otherwise, horizontally center the text.
  //Vertical
  //0x4 : Put text Below the coordinate.
  //0x8 : Put text Above the coordinate.
  //Otherwise, vertically center the text.

  if (sourceName.empty() || sourceName == "NULL_INDEX")
    return;

  map <string, SDLResourceFrameSource> :: iterator iter2;
  iter2 = frameSources.find (sourceName);

  if (iter2 == frameSources.end())
    return;

  SDLResourceFrameSource sourceToUse = (*iter2).second;

  if (respectCamera)
    {
      sourceToUse.setXPos (screenResolutionX/2 + xPos - observerX);
      sourceToUse.setYPos (screenResolutionY/2 + yPos - observerY);
    }
  else
    {
      sourceToUse.setXPos (xPos);
      sourceToUse.setYPos (yPos);
    }

  sourceToUse.draw (screen, orientation);
}

void SDLResourceEngine :: drawGFX (const string & gfxName, int32_t xPos, int32_t yPos, GFXMod * gm, bool respectCamera)
{
  //respectCamera : If false, xPos and yPos are screen coordinates and ignore camera scroll.

  //draws gfx at the given coordinates.
  if (gfxName.empty() || gfxName == "NULL_INDEX")
    return;

  map <string, SDLResourceGFX *> :: iterator iter2;
  iter2 = gfxSources.find (gfxName);

  if (iter2 == gfxSources.end())
    return;

  SDLResourceGFX * sourceToUse = (*iter2).second;

  if (respectCamera)
    {
      sourceToUse -> setXPos (screenResolutionX/2 + xPos - observerX);
      sourceToUse -> setYPos (screenResolutionY/2 + yPos - observerY);
    }
  else
    {
      sourceToUse -> setXPos (xPos);
      sourceToUse -> setYPos (yPos);
    }
  sourceToUse -> draw (screen, gm);
}

void SDLResourceEngine :: releaseFrameSource (const string & sourceName)
{
  //when the engine is finished with this resource it is released.
  if (sourceName.empty() || sourceName == "NULL_INDEX")
    return;

  map <string, SDLResourceFrameSource> :: iterator iter2;
  iter2 = frameSources.find (sourceName);

  if (iter2 == frameSources.end())
    return;
  
  //loop through and clear resources.
  if(iter2 != frameSources.end ()) {
    (*iter2).second.releaseFrameBuffer ();
    frameSources.erase(iter2); //now delete it.
  }
}

void SDLResourceEngine :: releaseSoundSource (const string & sourceName)
{
  //when the engine is finished with this resource it is released.
  if (sourceName.empty() || sourceName == "NULL_INDEX")
    return;

  map <string, SDLResourceSoundSource *> :: iterator iter2;
  iter2 = soundSources.find (sourceName);

  if (iter2 == soundSources.end())
    return;
 
  if(iter2 != soundSources.end ()) {
    (*iter2).second -> releaseSoundBuffer ();
    delete (*iter2).second;
    soundSources.erase(iter2); //now delete it.
  }
}

int SDLResourceEngine :: getGFXWidth (const string & gfxName)
{
  if (gfxName.empty() || gfxName == "NULL_INDEX")
    return 0;

  map <string, SDLResourceGFX *> :: iterator iter2;
  iter2 = gfxSources.find (gfxName);

  if (iter2 == gfxSources.end())
    return 0;

  return (*iter2).second -> getWidth ();
}

int SDLResourceEngine :: getGFXHeight (const string & gfxName)
{
  if (gfxName.empty() || gfxName == "NULL_INDEX")
    return 0;

  map <string, SDLResourceGFX *> :: iterator iter2;
  iter2 = gfxSources.find (gfxName);

  if (iter2 == gfxSources.end())
    return 0;

  return (*iter2).second -> getHeight ();
}

void SDLResourceEngine :: takeScreenshot ()
{
  takeScreenshotFlag = true;
}

void SDLResourceEngine :: flipScreen ()
{
  //flips the buffer.
  SDL_Flip(screen); //Double buffered, so I can simply call SDL_Flip

  time_t t;
  char * timeFormat;
  struct tm * timeinfo;
  uint32_t i;
  string filename;
  
  if (takeScreenshotFlag)
    {
      //construct a filename, copy to file
      time (&t); //stores the current time into t
      timeinfo = localtime (&t);
      timeFormat = asctime(timeinfo);
      filename.assign (timeFormat);
      i = filename.find_last_of('\n');
      if (i != string::npos)
	{
	  filename.erase (i, 1);
	}

      i = filename.find_first_of(' ');
      while (i != string::npos)
	{
	  filename.erase (i, 1);
	  filename.insert (i, "-");
	  i = filename.find_first_of(' ');
	}

      filename.append (".bmp");
      filename.insert (0, "screenshots/screenshot-");

      SDL_SaveBMP (screen, filename.c_str ());
      takeScreenshotFlag = false;
    }
}

void SDLResourceEngine :: clearScreen ()
{
  //clears the buffer
  SDL_Rect dest;
 
  dest.x = 0;
  dest.y = 0;
  dest.w = screenResolutionX;
  dest.h = screenResolutionY;

  uint32_t bkgColor = SDL_MapRGBA (screen -> format, bkgColorR, bkgColorG, bkgColorB, 0);

  SDL_FillRect(screen, &dest, bkgColor);
}

void SDLResourceEngine :: setBackgroundColor (uint8_t r, uint8_t g, uint8_t b)
{
  bkgColorR = r;
  bkgColorG = g;
  bkgColorB = b;
}

void SDLResourceEngine :: clearBackgroundColor ()
{
  setBackgroundColor (0, 0, 0);
}

int64_t SDLResourceEngine :: getObserverX ()
{
  return observerX;
}

int64_t SDLResourceEngine :: getObserverY ()
{
  return observerY;
}

void SDLResourceEngine :: setObserverX (int64_t obsx)
{
  observerX = obsx;
}

void SDLResourceEngine :: setObserverY (int64_t obsy)
{
  observerY = obsy;
}

uint16_t SDLResourceEngine :: getScreenResolutionX ()
{
  return screenResolutionX;
}

uint16_t SDLResourceEngine :: getScreenResolutionY ()
{
  return screenResolutionY;
}

void SDLResourceEngine :: setScreenResolution (uint16_t resX, uint16_t resY)
{
  screenResolutionX = resX;
  screenResolutionY = resY;
}

bool SDLResourceEngine :: loadMusic (const string & index)
{
  if (index.empty() || index == "NULL_INDEX")
    return false;

  musicIndex = "NULL_INDEX";

  if (musicResource -> loadMusic (index) == false)
    {
      writeToErrorLog ("loadMusic: music not loaded", 1);
      return false;
    }

  musicIndex = index;

  Mix_HookMusicFinished (&musicLooped);
  musicStartTime = SDL_GetTicks ();
  musicElapsedTime = 0;
  return true;
}

bool SDLResourceEngine :: loadIfNotLoadedMusic (const string & index)
{
  if (index.empty() || index == "NULL_INDEX")
    return false;

  if (index == musicIndex)
    return false;

  return loadMusic (index);
}

const string & SDLResourceEngine :: getMusicIndex ()
{
  return musicIndex;
}

void SDLResourceEngine :: playMusic (int loops, int ms, double position)
{
  //plays the already loaded song.
  //loops is the number of times to play the music. 0-1 to play once, 2 to play twice, and -1 to play infinitely. Default is 1.

  bool error = false; 
  SDL_ClearError ();
  error = musicResource -> PlayMusic (loops, ms, position);
  if (error == false)
    {
      writeToErrorLog ("playMusic: error while playing music", 1);
      writeToErrorLog (Mix_GetError(), 2);
    }

  musicStartTime = SDL_GetTicks ();
  return;
}

uint32_t SDLResourceEngine :: haltMusic (int ms)
{
  //if ms = 0, immediately stops the song, with no chance of failure.
  //if ms != 0, fades the song to silence, then halts the song. The fadeout takes ms milliseconds.
  //ms is overloaded to 0 if you leave it out.
  
  //returns elapsed time.

  musicResource -> HaltMusic (ms);

  return getMusicElapsedTime ();
}

void SDLResourceEngine :: freeMusic ()
{
  musicResource -> freeMusic ();
  musicIndex = "NULL_INDEX";
}

int SDLResourceEngine :: isPlayingMusic ()
{
  return musicResource -> isPlaying ();
}

void SDLResourceEngine :: loadFont (const string & fontIndex, string & keyName)
{
  if (fontIndex.empty() || fontIndex == "NULL_INDEX")
    {
      keyName = "NULL_INDEX";
      return;
    }

  if (fonts.size () >= SDL_RESOURCE_FONT_MAX)
    {
      writeToErrorLog ("loadFone: Maximum number of Fonts reached", 1);
      keyName = "NULL_INDEX";
      return;
    }

  SDLResourceFont * bufferToUse = NULL;
  bool loadBuffer = true;

  //if something already has this, return that.
  map <string, SDLResourceFont *> :: iterator iter2;
  iter2 = fonts.find (fontIndex);

  if (iter2 != fonts.end())
    {
      bufferToUse = (*iter2).second;
      loadBuffer = false;
    }

  if (loadBuffer)
    {
      bufferToUse = new SDLResourceFont ();
      if (bufferToUse -> loadFont (fontIndex) == false)
	{
	  writeToErrorLog ("loadFont: Font not loaded", 1);
	  delete bufferToUse;
	  keyName = "NULL_INDEX";
	  return;
	}
      fonts[fontIndex]=bufferToUse;
    }
  keyName=fontIndex;
  return;
}

void SDLResourceEngine :: drawText (const string & str, const string & fontIndex, int32_t xPos, int32_t yPos, uint8_t colorRed, uint8_t colorGreen, uint8_t colorBlue, bool respectCamera, uint8_t orientation)
{
  //respectCamera : If false, xPos and yPos are screen coordinates and ignore camera scroll.

  //orientation uses bit masks.
  //Horizontal
  //0x1 : Put text to the right of the coordinate.
  //0x2 : Put text to the left of the coordinate.
  //Otherwise, horizontally center the text.
  //Vertical
  //0x4 : Put text Below the coordinate.
  //0x8 : Put text Above the coordinate.
  //Otherwise, vertically center the text.

  if (fontIndex.empty() || fontIndex == "NULL_INDEX")
    return;

  map <string, SDLResourceFont *> :: iterator iter2;
  iter2 = fonts.find (fontIndex);

  if (iter2 == fonts.end())
    return;

  SDLResourceFont * sourceToUse = (*iter2).second;

  if (respectCamera)
    sourceToUse -> draw (str, screen, 
			 screenResolutionX/2 + xPos - observerX, 
			 screenResolutionY/2 + yPos - observerY,
			 colorRed, colorBlue, colorGreen, orientation);
  else
    sourceToUse -> draw (str, screen, xPos, yPos, colorRed, colorGreen, colorBlue, orientation);
}

uint16_t SDLResourceEngine :: getTextDimensions (const string & str, const string & fontIndex, uint16_t * widthReturn, uint16_t * heightReturn)
{
  //determine the width and height by drawing it.
  //returns the width. If widthReturn and heightReturn are not NULL, they also store the dimensions.

  if (fontIndex.empty() || fontIndex == "NULL_INDEX")
    return 0;

  map <string, SDLResourceFont *> :: iterator iter2;
  iter2 = fonts.find (fontIndex);

  if (iter2 == fonts.end())
    return 0;

  SDLResourceFont * sourceToUse = (*iter2).second;

  return sourceToUse -> getTextDimensions (str, widthReturn, heightReturn);
}

uint16_t SDLResourceEngine :: getFontSize (const string & fontIndex)
{
  if (fontIndex.empty() || fontIndex == "NULL_INDEX")
    return 0;

  map <string, SDLResourceFont *> :: iterator iter2;
  iter2 = fonts.find (fontIndex);

  if (iter2 == fonts.end())
    return 0;

  SDLResourceFont * sourceToUse = (*iter2).second;
  return sourceToUse -> getFontSize ();
}

bool SDLResourceEngine :: toonChooseAnim (uint16_t index, uint8_t anim)
{
  if (index == NULL_INDEX)
    return false;
  
  if (index > toons -> size ())
    return false;

  return toons -> at (index - 1) -> chooseAnim (anim);
}

void SDLResourceEngine :: toonPlayAnim (uint16_t index)
{
  if (index == NULL_INDEX)
    return;
  
  if (index > toons -> size ())
    return;

  toons -> at (index - 1) -> playAnim ();
}

void SDLResourceEngine :: toonPauseAnim (uint16_t index)
{
  if (index == NULL_INDEX)
    return;
  
  if (index > toons -> size ())
    return;
  
  toons -> at (index - 1) -> pauseAnim ();
}

void SDLResourceEngine :: toonStopAnim (uint16_t index)
{
  if (index == NULL_INDEX)
    return;
  
  if (index > toons -> size ())
    return;
  
  toons -> at (index - 1) -> stopAnim ();
}

void SDLResourceEngine :: toonDraw (uint16_t index, int32_t x, int32_t y)
{
  if (index == NULL_INDEX)
    return;
  
  if (index > toons -> size ())
    return;

  toons -> at (index - 1) -> draw (x, y);
}

uint32_t SDLResourceEngine :: getMusicElapsedTime ()
{
  uint32_t musLen = musicResource -> getMusicLength ();

  if (Mix_PlayingMusic () == 1 && Mix_PausedMusic () == 0)
    {
      if (musLen == 0)
	musicElapsedTime = SDL_GetTicks () - musicStartTime;
      else
	musicElapsedTime = (SDL_GetTicks () - musicStartTime) % musLen;
      return musicElapsedTime;
    }

  if (Mix_PlayingMusic () == 1 || Mix_PausedMusic () == 1)
    {
      if (musLen >= 0)
	musicElapsedTime = musicElapsedTime % musLen;
      return musicElapsedTime;
    }
  return 0;
}

double SDLResourceEngine :: getMusicElapsedTimeDouble ()
{
  return (double(getMusicElapsedTime ()) / 1000.0);
}

double SDLResourceEngine :: saveMusicElapsedTime ()
{
  musicPreviousElapsedTime = getMusicElapsedTimeDouble ();
  return musicPreviousElapsedTime;
}

double SDLResourceEngine :: getMusicPreviousElapsedTime ()
{
  return musicPreviousElapsedTime;
}

void SDLResourceEngine :: loadComplexGFX (const string & cgIndex, string & keyName)
{
  if (cgIndex.empty() || cgIndex == "NULL_INDEX") //0 is a null
    {
      keyName = "NULL_INDEX";
      return;
    }
  //Step 0: Find the first free source.
  if (cgSources.size () >= SDL_RESOURCE_COMPLEX_GFX_MAX)
    {
      writeToErrorLog ("loadComplexGFX: Maximum number of Complex GFX reached", 1);
      keyName = "NULL_INDEX";
      return;
    }

  SDLResourceComplexGFX * bufferToUse = NULL;
  bool loadBuffer = true;

  //if something already has this, return that.
  map <string, SDLResourceComplexGFX *> :: iterator iter2;
  iter2 = cgSources.find (cgIndex);

  if (iter2 != cgSources.end())
    {
      bufferToUse = (*iter2).second;
      loadBuffer = false;
    }

  if (loadBuffer)
    {
      bufferToUse = new SDLResourceComplexGFX ();

      if (bufferToUse -> loadComplexGFX (cgIndex, this) == false)
	{
	  writeToErrorLog ("loadComplexGFX: GFX not loaded", 1);
	  writeToErrorLog (cgIndex.c_str(), 2);
	  delete bufferToUse;
	  keyName = "NULL_INDEX";
	  return;
	}
      cgSources[cgIndex]=bufferToUse;
    }
  keyName = cgIndex;
  return;
}

void SDLResourceEngine :: drawComplexGFX (const string & cgIndex, int32_t xPos, int32_t yPos, GFXMod * gm, bool respectCamera)
{
  //respectCamera : If false, xPos and yPos are screen coordinates and ignore camera scroll.

  //draws complex gfx at the given coordinates.
  if (cgIndex.empty() || cgIndex == "NULL_INDEX") //0 is a null
    return;
  
  map <string, SDLResourceComplexGFX *> :: iterator iter2;
  iter2 = cgSources.find (cgIndex);

  if (iter2 == cgSources.end())
    return;

  SDLResourceComplexGFX * sourceToUse = (*iter2).second;

  if (respectCamera)
    {
      sourceToUse -> setXPos (screenResolutionX/2 + xPos - observerX);
      sourceToUse -> setYPos (screenResolutionY/2 + yPos - observerY);
    }
  else
    {
      sourceToUse -> setXPos (xPos);
      sourceToUse -> setYPos (yPos);
    }
  sourceToUse -> draw (screen, gm, this);
}

void musicLooped ()
{
  //this is called when the music loops.

  musicStartTime = SDL_GetTicks ();
  musicElapsedTime = 0;

  //when you implement pausing:
  //Add (now - startTime) seconds to elapsedTime
}
