#ifndef SDLRESOURCEENGINE_H
#define SDLRESOURCEENGINE_H

#include <stdint.h>

#include "SDL.h"
#include "SDL_ttf.h"

#include "SDLResourceFrameBuffer.h"
#include "SDLResourceFrameSource.h"
#include "SDLResourceGFX.h"
#include "SDLResourceMusic.h"
#include "SDLResourceSoundBuffer.h"
#include "SDLResourceSoundSource.h"
#include "SDLResourceToon.h"
#include "SDLResourceFont.h"
#include "SDLResourceComplexGFX.h"
#include "GFXMod.h"

#include <vector>
#include <string>
#include <map>
using namespace std;

#define NULL_INDEX 0
#define SDL_RESOURCE_FRAME_SOURCE_MAX 1024
#define SDL_RESOURCE_SOUND_SOURCE_MAX 1024
#define SDL_RESOURCE_GFX_MAX 1024
#define SDL_RESOURCE_COMPLEX_GFX_MAX 1024
#define SDL_RESOURCE_TOON_MAX 1024
#define SDL_RESOURCE_FONT_MAX 1024

#define RESOURCE_TYPE_FRAME 0

#define DRAW_ANCHOR_CENTER 0
#define DRAW_ANCHOR_RIGHT 2
#define DRAW_ANCHOR_LEFT 1
#define DRAW_ANCHOR_DOWN 8
#define DRAW_ANCHOR_UP 4
#define DRAW_ANCHOR_DOWN_RIGHT 10
#define DRAW_ANCHOR_DOWN_LEFT 9
#define DRAW_ANCHOR_UP_RIGHT 6
#define DRAW_ANCHOR_UP_LEFT 5

//Here's the SDLResourceEngine. Try to keep a lone instance of this.

class SDLResourceEngine
{
 public:
  SDLResourceEngine();
  ~SDLResourceEngine();

  int8_t SDLStartup ();

  void loadFrame (const string &, string &); //loads a frame into a buffer, adds a source, assigns a source to the buffer, then returns a reference to the new source. returns 0 on error.
  void drawFrame (const string &, int32_t, int32_t, bool respectCamera = false, uint8_t orientation = 0);
  void releaseFrameSource (const string &); //when the engine is finished with this resource it is released.

  void loadSound (const string &, string &); //loads a sound into a buffer, adds a source, assigns a source to the buffer, then returns a reference to the new source. returns 0 on error.
  void playSound (const string &, uint8_t volume = MIX_MAX_VOLUME, uint8_t leftPan = 0); //plays the sound effect with volume and panning parameters.
  void releaseSoundSource (const string &); //when the engine is finished with this resource it is released.
 
  void loadGFX (const string &, string &); //loads graphic effects into a buffer
  void drawGFX (const string &, int32_t, int32_t, GFXMod * gm = NULL, bool respectCamera = true);
  void releaseGFX (const string &);
  int getGFXWidth (const string &);
  int getGFXHeight (const string &);

  bool loadMusic (const string &); //loads a song to be played.
  bool loadIfNotLoadedMusic (const string &); //loads a song if this song isn't the current song.
  void playMusic (int loops = 1, int ms = 0, double position = 0.0);
  uint32_t haltMusic (int ms = 0); //halts a song, or fades out.
  void freeMusic ();
  int isPlayingMusic ();
  const string & getMusicIndex ();
  uint32_t getMusicElapsedTime ();
  double getMusicElapsedTimeDouble ();
  double saveMusicElapsedTime ();
  double getMusicPreviousElapsedTime ();

  void loadFont (const string &, string &);
  void drawText (const string & str, const string & fontIndex, int32_t xPos, int32_t yPos, uint8_t colorRed = 176, uint8_t colorGreen = 176, uint8_t colorBlue = 176, bool respectCamera = false, uint8_t orientation = 0);
  uint16_t getFontSize (const string & fontIndex);
  uint16_t getTextDimensions (const string & str, const string &, uint16_t * widthReturn, uint16_t * heightReturn);

  void flipScreen ();
  void clearScreen ();
  void setBackgroundColor (uint8_t r, uint8_t g, uint8_t b);
  void clearBackgroundColor ();
  void takeScreenshot ();

  int64_t getObserverX ();
  int64_t getObserverY ();

  void setObserverX (int64_t);
  void setObserverY (int64_t);

  uint16_t getScreenResolutionX ();
  uint16_t getScreenResolutionY ();

  void setScreenResolution (uint16_t, uint16_t);

  uint16_t loadToon (const string &);

  bool toonChooseAnim (uint16_t index, uint8_t anim);
  void toonPlayAnim (uint16_t index);
  void toonPauseAnim (uint16_t index);
  void toonStopAnim (uint16_t index);
  void toonDraw (uint16_t index, int32_t x, int32_t y);

  void loadComplexGFX (const string & cgIndex, string & keyName);
  void drawComplexGFX (const string &, int32_t, int32_t, GFXMod * gm = NULL, bool respectCamera = true);

 private:
  uint16_t screenResolutionX;
  uint16_t screenResolutionY;

  uint16_t minScreenResolutionX;
  uint16_t minScreenResolutionY;
  bool windowedScreen;

  int64_t observerX;
  int64_t observerY;

  SDL_Surface * screen;
  uint8_t bkgColorR;
  uint8_t bkgColorG;
  uint8_t bkgColorB;
  bool takeScreenshotFlag;

  vector <SDLResourceFrameBuffer> frameBuffers;
  map <string, SDLResourceFrameSource> frameSources;

  vector <SDLResourceSoundBuffer *> soundBuffers;
  map <string, SDLResourceSoundSource *> soundSources;

  map <string, SDLResourceGFX *> gfxSources;
  map <string, SDLResourceComplexGFX *> cgSources;

  map <string, SDLResourceFont *> fonts;

  vector <SDLResourceToon *> * toons;

  SDLResourceMusic * musicResource;
  string musicIndex;

  int audioFreq;
  Uint16 audioFormat;
  int audioChannels;

  uint32_t loadCounter; //running count of the number of times a load function was called.
};

void musicLooped ();
static volatile uint32_t musicStartTime;
static volatile uint32_t musicElapsedTime;
static volatile double musicPreviousElapsedTime;
#endif
