#include <stdint.h>

#include "SDL.h"
#include "SDLResourceFont.h"
#include "SDL_ttf.h"
#include "../fileUtils.h"

using namespace std;

SDLResourceFont :: SDLResourceFont ()
{
  font = NULL;
  fontRW = NULL;
  fontIndex = "NULL_INDEX";
  fontSize = 0;
}

SDLResourceFont :: ~SDLResourceFont ()
{
  if (fontRW != NULL)
    {
      TTF_CloseFont (font);
      SDL_RWclose(fontRW);
    }
  fontRW = NULL;
  font = NULL;
  fontIndex = "NULL_INDEX";
}

bool SDLResourceFont :: loadFont (const string & index)
{
  //loads the font file
  if (index.empty() || index == "NULL_INDEX")
    return false;

  if (fontIndex != "NULL_INDEX")
    return false;

  //parse the xml file and find the filename for the given index
  
  xmlDocPtr doc;
  xmlXPathContextPtr xpathCtx;

  if (!loadXMLFile (doc, "resources/FontManifest.xml"))
    {
      xmlFreeDoc(doc);
      return false;
    }

  xpathCtx = xmlXPathNewContext(doc);
  if(xpathCtx == NULL) {
    writeToErrorLog("Error: unable to create new XPath context",0);
    xmlFreeDoc(doc);
    return false;
  }
  
  xmlChar *keyword;
  keyword = getFirstElementUsingXPath (doc, xpathCtx, "/FontManifest/Font[@ID='", index, "']/Filename");

  if (keyword==NULL)
    {
      xmlXPathFreeContext(xpathCtx);
      xmlFreeDoc(doc);
      return false;
    }

  fontRW = SDL_RWFromFile ((const char *)keyword, "r");
  xmlFree(keyword);
  keyword = getFirstElementUsingXPath (doc, xpathCtx, "/FontManifest/Font[@ID='", index, "']/Size");

  if (keyword!=NULL)
    {
      fontSize=int(xmlXPathCastStringToNumber(keyword));
      xmlFree(keyword);
    }
  else
    {
      fontSize=14;
    }

  if (fontRW == NULL)
    {
      writeToErrorLog ("fontRW did not load, probably the file doesn't exist.", 1);
      writeToErrorLog (SDL_GetError(), 2);

      xmlXPathFreeContext(xpathCtx);
      xmlFreeDoc(doc);
      return false;
    }

  SDL_ClearError ();
  font = TTF_OpenFontRW (fontRW, 0, fontSize);
  if (font == NULL)
    {
      writeToErrorLog ("loadFont: font not loaded for index", 1);
      writeToErrorLog (index.c_str(), 1);
      writeToErrorLog (TTF_GetError(), 2);

      xmlXPathFreeContext(xpathCtx);
      xmlFreeDoc(doc);
      return false;
    }

  xmlXPathFreeContext(xpathCtx);
  xmlFreeDoc(doc);
  
  fontIndex = index;
  return true;
}

void SDLResourceFont :: draw (const string & str, SDL_Surface * output, int32_t xPos, int32_t yPos, uint8_t colorRed, uint8_t colorGreen, uint8_t colorBlue, uint8_t orientation)
{
  //orientation uses bit masks.
  //Horizontal
  //0x1 : Put text to the right of the coordinate.
  //0x2 : Put text to the left of the coordinate.
  //Otherwise, horizontally center the text.
  //Vertical
  //0x4 : Put text Below the coordinate.
  //0x8 : Put text Above the coordinate.
  //Otherwise, vertically center the text.

  SDL_Color textCol;
  textCol.r = colorRed;
  textCol.g = colorGreen;
  textCol.b = colorBlue;

  SDL_Rect tPos;
  tPos.x = xPos;
  tPos.y = yPos;

  SDL_Surface * textOutput;
  textOutput = TTF_RenderText_Solid (font, str.c_str (), textCol);

  if (orientation & 0x1)
    tPos.x += 0;
  else if (orientation & 0x2)
    tPos.x -= textOutput -> w;
  else
    tPos.x -= textOutput -> w / 2;
  
  if (orientation & 0x4)
    tPos.y += 0;
  else if (orientation & 0x8)
    tPos.y -= textOutput -> h;
  else
    tPos.y -= textOutput -> h / 2;

  SDL_BlitSurface(textOutput, NULL, output, &tPos);
  SDL_FreeSurface (textOutput);
}

uint16_t SDLResourceFont :: getFontSize ()
{
  return fontSize;
}

uint16_t SDLResourceFont :: getTextDimensions (const string & str, uint16_t * widthReturn, uint16_t * heightReturn)
{
  int wadth, hight;

  SDL_Color textCol;
  textCol.r = 128;
  textCol.g = 128;
  textCol.b = 128;

  SDL_Surface * textOutput;
  textOutput = TTF_RenderText_Solid (font, str.c_str (), textCol);

  wadth = textOutput -> w;
  hight = textOutput -> h;

  SDL_FreeSurface (textOutput);

  if (widthReturn != NULL)
    (*widthReturn) = (uint16_t)(wadth);

  if (heightReturn != NULL)
    (*heightReturn) = (uint16_t)(hight);

  return (uint16_t)(wadth);
}
