#include <stdint.h>
#include "SDL.h"
#include "SDLResourceGFX.h"
#include "SDL_gfxPrimitives.h"
#include "../fileUtils.h"

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#include "GFXMod.h"

using namespace std;

SDLResourceGFX :: SDLResourceGFX ()
{
  color = 0;
  type = 0;
  w = 0, h = 0;
  dimX = new vector<int32_t> (0);
  dimY = new vector<int32_t> (0);
  gfxIndex = "NULL_INDEX";
}

SDLResourceGFX :: ~SDLResourceGFX()
{
  if (dimX != NULL)
    delete dimX;
  
  if (dimY != NULL)
    delete dimY;
}

bool SDLResourceGFX :: loadGFX (const string & index)
{
  if (index == "NULL_INDEX")
    return false; //Null index

  xmlDocPtr doc;
  xmlXPathContextPtr xpathCtx;

  if (!loadXMLFile (doc, "resources/GFXManifest.xml"))
    {
      xmlFreeDoc(doc);
      return false;
    }

  xpathCtx = xmlXPathNewContext(doc);
  if(xpathCtx == NULL) {
    writeToErrorLog("Error: unable to create new XPath context",0);
    xmlFreeDoc(doc); 
    return false;
  }

  //just jump straight to load shape.

  string xpathStart="/GFXManifest/GFX[@ID='";
  xpathStart+=index;
  xpathStart+="']";

  bool tempBool = false;
  tempBool = loadShape (doc, xpathCtx, xpathStart);

  xmlXPathFreeContext(xpathCtx);
  xmlFreeDoc(doc);

  if (!tempBool)
    {
      //Failed to load shape. Cleanup and exit.
      writeToErrorLog ("Failed to load GFX for index", 0);
      writeToErrorLog (index.c_str(), 1);
      return false;
    }

  //run a function to calculate the width and height
  calculateDimensions ();

  //set the pic index
  gfxIndex = index;

  return true;
}

bool SDLResourceGFX :: loadShape (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  //loads the shape, given an xml document doc, a context pointer and the xpath to identify the shape record. This populates the sdlResourceGFX.

  type = GFX_TYPE_NULL;

  xmlChar * element;
  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Shape", "");  

  if (element == NULL)
    {
    return false;
    }
  if (xmlStrcasecmp (element, (const xmlChar *)"pixel") == 0)
    type = GFX_TYPE_PIXEL;
  if (xmlStrcasecmp (element, (const xmlChar *)"circle") == 0)
    type = GFX_TYPE_CIRCLE;
  if (xmlStrcasecmp (element, (const xmlChar *)"filledcircle") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"circlefilled") == 0)
    type = GFX_TYPE_CIRCLE_FILLED;
  if (xmlStrcasecmp (element, (const xmlChar *)"aacircle") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"circleaa") == 0)
    type = GFX_TYPE_CIRCLE_AA;
  if (xmlStrcasecmp (element, (const xmlChar *)"pie") == 0)
    type = GFX_TYPE_PIE;
  if (xmlStrcasecmp (element, (const xmlChar *)"filledpie") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"piefilled") == 0)
    type = GFX_TYPE_PIE_FILLED;
  if (xmlStrcasecmp (element, (const xmlChar *)"ellipse") == 0)
    type = GFX_TYPE_ELLIPSE;
  if (xmlStrcasecmp (element, (const xmlChar *)"filledellipse") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"ellipsefilled") == 0)
    type = GFX_TYPE_ELLIPSE_FILLED;
  if (xmlStrcasecmp (element, (const xmlChar *)"aaellipse") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"ellipseaa") == 0)
    type = GFX_TYPE_ELLIPSE_AA;
  if (xmlStrcasecmp (element, (const xmlChar *)"horizontalline") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"horizline") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"horiz line") == 0)
    type = GFX_TYPE_HORIZONTAL_LINE;
  if (xmlStrcasecmp (element, (const xmlChar *)"verticalline") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"vertline") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"vert line") == 0)
    type = GFX_TYPE_VERTICAL_LINE;
  if (xmlStrcasecmp (element, (const xmlChar *)"line") == 0)
    type = GFX_TYPE_LINE;
  if (xmlStrcasecmp (element, (const xmlChar *)"aaline") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"lineaa") == 0)
    type = GFX_TYPE_LINE_AA;
  if (xmlStrcasecmp (element, (const xmlChar *)"rectangle") == 0)
    type = GFX_TYPE_RECTANGLE;
  if (xmlStrcasecmp (element, (const xmlChar *)"filledrectangle") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"rectanglefilled") == 0)
    type = GFX_TYPE_RECTANGLE_FILLED;
  if (xmlStrcasecmp (element, (const xmlChar *)"trigon") == 0)
    type = GFX_TYPE_TRIGON;
  if (xmlStrcasecmp (element, (const xmlChar *)"trigonaa") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"aatrigon") == 0)
    type = GFX_TYPE_TRIGON_AA;
  if (xmlStrcasecmp (element, (const xmlChar *)"trigonfilled") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"filledtrigon") == 0)
    type = GFX_TYPE_TRIGON_FILLED;
  if (xmlStrcasecmp (element, (const xmlChar *)"polygon") == 0)
    type = GFX_TYPE_POLYGON;
  if (xmlStrcasecmp (element, (const xmlChar *)"polygonfilled") == 0 ||
      xmlStrcasecmp (element, (const xmlChar *)"filledpolygon") == 0)
    type = GFX_TYPE_POLYGON_FILLED;
  if (xmlStrcasecmp (element, (const xmlChar *)"beziercurve") == 0)
    type = GFX_TYPE_BEZIER_CURVE;

  xmlFree (element);

  int tempCol;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/ColorRed", "");
  if (element == NULL)
    tempCol = 0;
  else
    tempCol = int(xmlXPathCastStringToNumber(element));
  
  if (tempCol < 0)
    tempCol = 0;
  if (tempCol > 255)
    tempCol = 255;
  color += (tempCol << 24);
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/ColorGreen", "");
  if (element == NULL)
    tempCol = 0;
  else
    tempCol = int(xmlXPathCastStringToNumber(element));
  
  if (tempCol < 0)
    tempCol = 0;
  if (tempCol > 255)
    tempCol = 255;
  color += (tempCol << 16);
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/ColorBlue", "");
  if (element == NULL)
    tempCol = 0;
  else
    tempCol = int(xmlXPathCastStringToNumber(element));
  
  if (tempCol < 0)
    tempCol = 0;
  if (tempCol > 255)
    tempCol = 255;
  color += (tempCol << 8);
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/ColorAlpha", "");
  if (element == NULL)
    tempCol = 255;
  else
    tempCol = int(xmlXPathCastStringToNumber(element));
  
  if (tempCol < 0)
    tempCol = 0;
  if (tempCol > 255)
    tempCol = 255;
  color += tempCol;
  xmlFree (element);

  //send to loading subsection, based on shape type
  switch (type)
    {
    case(GFX_TYPE_NULL):
    case(GFX_TYPE_PIXEL):
      break;
    case(GFX_TYPE_PIE):
    case(GFX_TYPE_PIE_FILLED):
      return loadShapePie (doc, xpathCtx, xpath);
      break;
    case (GFX_TYPE_BEZIER_CURVE):
      return loadShapeBezier (doc, xpathCtx, xpath);
      break;
    case(GFX_TYPE_POLYGON):
    case(GFX_TYPE_POLYGON_FILLED):
      return loadShapePolygon (doc, xpathCtx, xpath);
      break;
    case(GFX_TYPE_VERTICAL_LINE):
      return loadShapeVertLine (doc, xpathCtx, xpath);
      break;
    case(GFX_TYPE_HORIZONTAL_LINE):
      return loadShapeHorizLine (doc, xpathCtx, xpath);
      break;
    case(GFX_TYPE_ELLIPSE):
    case(GFX_TYPE_ELLIPSE_FILLED):
    case(GFX_TYPE_ELLIPSE_AA):
      return loadShapeEllipse (doc, xpathCtx, xpath);
      break;
    case(GFX_TYPE_CIRCLE):
    case(GFX_TYPE_CIRCLE_FILLED):
    case(GFX_TYPE_CIRCLE_AA):
      return loadShapeCircle (doc, xpathCtx, xpath);
      break;
    case(GFX_TYPE_TRIGON):
    case(GFX_TYPE_TRIGON_AA):
    case(GFX_TYPE_TRIGON_FILLED):
      return loadShapeTrigon (doc, xpathCtx, xpath);
      break;
    case(GFX_TYPE_LINE):
    case(GFX_TYPE_LINE_AA):
    case(GFX_TYPE_RECTANGLE):
    case(GFX_TYPE_RECTANGLE_FILLED):
      return loadShapeLineOrRectangle (doc, xpathCtx, xpath);
      break;
    default:
      writeToErrorLog ("Invalid shape type", 0);
      return false;
      break;
    };
  return true;
}

bool SDLResourceGFX :: loadShapeCircle (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  //loads data for a circle, filled circle, or AA circle.
  xmlChar * element;
  int temp;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Radius", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimX -> push_back ((int32_t)(temp));

  xmlFree (element);
  return true;
}

bool SDLResourceGFX :: loadShapePie (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  //loads data for a pie or filled pie.
  xmlChar * element;
  int temp;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Radius", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimX -> push_back ((int32_t)(temp));
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/StartAngle", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimX -> push_back ((int32_t)(temp));
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/EndAngle", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimX -> push_back ((int32_t)(temp));
  xmlFree (element);
  return true;
}

bool SDLResourceGFX :: loadShapeEllipse (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  xmlChar * element;
  int temp;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/RadiusX", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimX -> push_back ((int32_t)(temp));
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/RadiusY", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimY -> push_back ((int32_t)(temp));
  xmlFree (element);
  return true;
}

bool SDLResourceGFX :: loadShapeHorizLine (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  xmlChar * element;
  int temp;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Width", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimX -> push_back ((int32_t)(temp));

  xmlFree (element);
  return true;
}

bool SDLResourceGFX :: loadShapeVertLine (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  xmlChar * element;
  int temp;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Height", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimY -> push_back ((int32_t)(temp));

  xmlFree (element);
  return true;
}

bool SDLResourceGFX :: loadShapeLineOrRectangle (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  xmlChar * element;
  int temp;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Width", "");
  if (element == NULL)
    {
    return false;
    }
  temp = int(xmlXPathCastStringToNumber(element));
  dimX -> push_back ((int32_t)(temp));
  xmlFree (element);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Height", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimY -> push_back ((int32_t)(temp));
  xmlFree (element);
  return true;
}

bool SDLResourceGFX :: loadShapeTrigon (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  xmlChar * element;
  xmlChar * secondElement;
  xmlChar * thirdElement;
  int temp;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Point1", "");
  if (element == NULL)
    return false;

 //find the first space
  if (xmlStrchr (element, (const xmlChar)' ') == NULL && xmlStrchr (element, (const xmlChar)',') == NULL)
    return false;

  if (xmlStrchr (element, (const xmlChar)' ') == NULL)
    thirdElement = xmlStrndup (element, xmlStrchr(element, (const xmlChar)',')-element);
  else
    thirdElement = xmlStrndup (element, xmlStrchr(element, (const xmlChar)' ')-element);

  temp = int(xmlXPathCastStringToNumber(thirdElement));
  dimX -> push_back ((int32_t)(temp));

  if (xmlStrchr (element, (const xmlChar)' ') == NULL)
    secondElement = xmlStrdup (xmlStrchr(element, (const xmlChar)','));
  else
    secondElement = xmlStrdup (xmlStrchr(element, (const xmlChar)' '));

  temp = int(xmlXPathCastStringToNumber(secondElement+1)); //skip the first character, usualy a + or ,
  dimY -> push_back ((int32_t)(temp));

  xmlFree (element);
  xmlFree (secondElement);
  xmlFree (thirdElement);

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Point2", "");
  if (element == NULL)
    return false;

  if (xmlStrchr (element, (const xmlChar)' ') == NULL)
    thirdElement = xmlStrndup (element, xmlStrchr(element, (const xmlChar)',')-element);
  else
    thirdElement = xmlStrndup (element, xmlStrchr(element, (const xmlChar)' ')-element);

  temp = int(xmlXPathCastStringToNumber(thirdElement));
  dimX -> push_back ((int32_t)(temp));

 //find the first space
  if (xmlStrchr (element, (const xmlChar)' ') == NULL && xmlStrchr (element, (const xmlChar)',') == NULL)
    return false;

  if (xmlStrchr (element, (const xmlChar)' ') == NULL)
    secondElement = xmlStrdup (xmlStrchr(element, (const xmlChar)','));
  else
    secondElement = xmlStrdup (xmlStrchr(element, (const xmlChar)' '));

  temp = int(xmlXPathCastStringToNumber(secondElement+1)); //skip the first character, usualy a + or ,
  dimY -> push_back ((int32_t)(temp));

  xmlFree (element);
  xmlFree (secondElement);
  xmlFree (thirdElement);
  return true;
}

bool SDLResourceGFX :: loadShapeBezier (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  if (!loadShapePolygon(doc, xpathCtx, xpath))
    return false;

  //that will load everything except s, the number of steps to render

  xmlChar * element;
  int temp;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/S", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimY -> at (0) = (int32_t)(temp);
  xmlFree (element);  

  return true;
}

bool SDLResourceGFX :: loadShapePolygon (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  xmlChar * element;
  xmlChar * secondElement;
  xmlChar * thirdElement;
  char searchString [255];
  int temp, numPoints, index;
  string tempString;

  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Points", "");
  if (element == NULL)
    return false;

  temp = int(xmlXPathCastStringToNumber(element));
  dimX -> push_back ((int32_t)(temp));
  dimY -> push_back (0); //padding
  xmlFree (element);

  numPoints = temp;
  index = 0;

  if (numPoints < 3) //need at least 3 points to form a polygon
    return false;

  //extract points
  for (index = 1; index < numPoints; index++)
    {
      sprintf (searchString, "/Point%u", index);

      tempString=searchString;
      element = getFirstElementUsingXPath (doc, xpathCtx, xpath, tempString, "");
      if (element == NULL)
	{
	  break;
	}

      //find the first space
      if (xmlStrchr (element, (const xmlChar)' ') == NULL && xmlStrchr (element, (const xmlChar)',') == NULL)
	{
	  break;
	}

      if (xmlStrchr (element, (const xmlChar)' ') == NULL)
	thirdElement = xmlStrndup (element, xmlStrchr(element, (const xmlChar)',')-element);
      else
	thirdElement = xmlStrndup (element, xmlStrchr(element, (const xmlChar)' ')-element);

      temp = int(xmlXPathCastStringToNumber(thirdElement));
      dimX -> push_back ((int32_t)(temp));

      if (xmlStrchr (element, (const xmlChar)' ') == NULL)
	secondElement = xmlStrdup (xmlStrchr(element, (const xmlChar)','));
      else
	secondElement = xmlStrdup (xmlStrchr(element, (const xmlChar)' '));
      
      temp = int(xmlXPathCastStringToNumber(secondElement+1)); //skip the first character, usualy a + or ,
      dimY -> push_back ((int32_t)(temp));

      xmlFree (element);
      xmlFree (secondElement);
      xmlFree (thirdElement);
    }

  if (index < 3)
    return false; //not enough points to form a polygon

  return true;
}

int SDLResourceGFX :: getWidth ()
{
  return w;
}

int SDLResourceGFX :: getHeight ()
{
  return h;
}

int SDLResourceGFX :: getCenterOfMassX ()
{
  return centerOfMassX;
}

int SDLResourceGFX :: getCenterOfMassY ()
{
  return centerOfMassY;
}

void SDLResourceGFX :: setXPos (int32_t x)
{xPos = x;}

void SDLResourceGFX :: setYPos (int32_t y)
{yPos = y;}

const string & SDLResourceGFX :: getGFXIndex ()
{
  return gfxIndex;
}

uint32_t SDLResourceGFX :: getColor ()
{
  return color;
}

int16_t SDLResourceGFX :: getRadius ()
{
  return dimX -> at (0);
}

void SDLResourceGFX :: draw (SDL_Surface * output, GFXMod * gm)
{
  //based on type, draw to surface

  int16_t xArray [32];
  int16_t yArray [32];
  uint8_t counter;
  vector <int32_t> :: iterator xIter;
  vector <int32_t> :: iterator yIter;
  
  int32_t wdth, hght;

 switch (type)
   {
   case(GFX_TYPE_NULL):
     break;
   case(GFX_TYPE_PIXEL):
     pixelColor(output, xPos, yPos, getColor ());
     break;
   case(GFX_TYPE_CIRCLE):
     circleColor(output, xPos, yPos, getRadius (), getColor ());
     break;
   case(GFX_TYPE_CIRCLE_FILLED):
     filledCircleColor(output, xPos, yPos, getRadius (), getColor ());
     break;
   case(GFX_TYPE_CIRCLE_AA):
     aacircleColor(output, xPos, yPos, getRadius (), getColor ());
     break;
   case(GFX_TYPE_ELLIPSE):
     ellipseColor(output, xPos, yPos, dimX -> at (0), dimY -> at (0), getColor ());
     break;
   case(GFX_TYPE_ELLIPSE_FILLED):
     filledEllipseColor(output, xPos, yPos, dimX -> at (0), dimY -> at (0), getColor ());
     break;
   case(GFX_TYPE_ELLIPSE_AA):
     aaellipseColor(output, xPos, yPos, dimX -> at (0), dimY -> at (0), getColor ());
     break;
   case(GFX_TYPE_PIE):
     pieColor(output, xPos, yPos, dimX -> at (0), dimX -> at (1), dimX -> at (2), getColor ());
     break;
   case(GFX_TYPE_PIE_FILLED):
     filledPieColor(output, xPos, yPos, dimX -> at (0), dimX -> at (1), dimX -> at (2), getColor ());
     break;
   case(GFX_TYPE_HORIZONTAL_LINE):
     hlineColor(output, xPos - (w/2), xPos + (w/2), yPos, getColor ());
     break;
   case(GFX_TYPE_VERTICAL_LINE):
     vlineColor(output, xPos, yPos - (h/2), yPos + (h/2), getColor ());
     break;
   case(GFX_TYPE_LINE):
      lineColor(output, xPos - (dimX->at(0)/2), yPos - (dimY->at(0)/2), xPos + (dimX->at(0)/2), yPos + (dimY->at(0)/2), getColor ());
     break;
   case(GFX_TYPE_LINE_AA):
     aalineColor(output, xPos - (dimX->at(0)/2), yPos - (dimY->at(0)/2), xPos + (dimX->at(0)/2), yPos + (dimY->at(0)/2), getColor ());
     break;
   case(GFX_TYPE_RECTANGLE):
     wdth = (w);
     hght = (h);

     if (gm != NULL)
       {
	 wdth = gm -> newWidth;
	 hght = gm -> newHeight;
       }

     rectangleColor(output, xPos - (wdth/2), yPos - (hght/2), xPos + (wdth/2), yPos + (hght/2), getColor ());
     break;
   case(GFX_TYPE_RECTANGLE_FILLED):
     wdth = (w);
     hght = (h);

     if (gm != NULL)
       {
	 wdth = gm -> newWidth;
	 hght = gm -> newHeight;
       }

     boxColor(output, xPos - (wdth/2), yPos - (hght/2), xPos + (wdth/2), yPos + (hght/2), getColor ());
     break;
   case(GFX_TYPE_TRIGON):
     trigonColor(output, xPos + centerOfMassX - (w/2), 
		 yPos + centerOfMassY - (h/2), 
		 xPos + centerOfMassX - (w/2) + dimX -> at (0), 
		 yPos + centerOfMassY - (h/2) + dimY -> at (0), 
		 xPos + centerOfMassX - (w/2) + dimX -> at (1), 
		 yPos + centerOfMassY - (h/2) + dimY -> at (1), getColor());
     break;
   case(GFX_TYPE_TRIGON_AA):
     aatrigonColor(output, xPos + centerOfMassX - (w/2), 
		   yPos + centerOfMassY - (h/2), 
		   xPos + centerOfMassX - (w/2) + dimX -> at (0), 
		   yPos + dimY -> at (0) + centerOfMassY - (h/2), 
		   xPos + centerOfMassX - (w/2) + dimX -> at (1), 
		   yPos + dimY -> at (1) + centerOfMassY - (h/2), getColor());
     break;
   case(GFX_TYPE_TRIGON_FILLED):
     filledTrigonColor(output, xPos + centerOfMassX - (w/2), 
		       yPos + centerOfMassY - (h/2), 
		       xPos + centerOfMassX - (w/2) + dimX -> at (0), 
		       yPos + dimY -> at (0) + centerOfMassY - (h/2), 
		       xPos + centerOfMassX - (w/2) + dimX -> at (1), 
		       yPos + dimY -> at (1) + centerOfMassY - (h/2), getColor());
     break;
   case(GFX_TYPE_BEZIER_CURVE):
     counter = 1;
     xIter = dimX -> begin () + 1;
     yIter = dimY -> begin () + 1;

     xArray [0] = xPos + centerOfMassX - (w/2);
     yArray [0] = yPos + centerOfMassY - (h/2);

     while (xIter != dimX -> end () && yIter != dimY -> end () && counter < dimX -> size () && counter < dimY -> size ())
       {
	 xArray[counter] = (*xIter) + xPos + centerOfMassX - (w/2);
	 yArray[counter] = (*yIter) + yPos + centerOfMassY - (h/2);

	 counter ++;
	 xIter++;
	 yIter++;
       }

     bezierColor(output, xArray, yArray, counter, dimY -> at (0), getColor ());
     break;
   case(GFX_TYPE_POLYGON):
     counter = 1;
     xIter = dimX -> begin () + 1;
     yIter = dimY -> begin () + 1;

     xArray [0] = xPos + centerOfMassX - (w/2);
     yArray [0] = yPos + centerOfMassY - (h/2);

     while (xIter != dimX -> end () && yIter != dimY -> end () && counter < dimX -> size () && counter < dimY -> size ())
       {
	 xArray[counter] = (*xIter) + xPos + centerOfMassX - (w/2);
	 yArray[counter] = (*yIter) + yPos + centerOfMassY - (h/2);

	 counter ++;
	 xIter++;
	 yIter++;
       }
     
     polygonColor(output, xArray, yArray, counter, getColor ());
     break;
   case(GFX_TYPE_POLYGON_FILLED):
     counter = 1;
     xIter = dimX -> begin () + 1;
     yIter = dimY -> begin () + 1;

     xArray [0] = xPos + centerOfMassX - (w/2);
     yArray [0] = yPos + centerOfMassY - (h/2);

     while (xIter != dimX -> end () && yIter != dimY -> end () && counter < dimX -> size () && counter < dimY -> size ())
       {
	 xArray[counter] = (*xIter) + xPos + centerOfMassX - (w/2);
	 yArray[counter] = (*yIter) + yPos + centerOfMassY - (h/2);

	 counter ++;
	 xIter++;
	 yIter++;
       }
     
     filledPolygonColor(output, xArray, yArray, counter, getColor ());
     break;
   default:
     break;
  };
}

void SDLResourceGFX :: calculateDimensions ()
{
  int32_t minX = 0, minY = 0, maxX = 0, maxY = 0;
  vector <int32_t> :: iterator xIter;
  vector <int32_t> :: iterator yIter;

  switch (type)
    {
    case(GFX_TYPE_NULL):
      w = 0; h = 0;
      break;
    case(GFX_TYPE_PIXEL):
      w = 0; h = 0;
      break;
    case(GFX_TYPE_CIRCLE):
    case(GFX_TYPE_CIRCLE_FILLED):
    case(GFX_TYPE_CIRCLE_AA):
      w = dimX -> at (0); h = w;
      break;
    case(GFX_TYPE_ELLIPSE):
    case(GFX_TYPE_ELLIPSE_FILLED):
    case(GFX_TYPE_ELLIPSE_AA):
    case(GFX_TYPE_PIE):
    case(GFX_TYPE_PIE_FILLED):
      w = dimX -> at (0); h = w;
      break;
    case(GFX_TYPE_HORIZONTAL_LINE):
      h = 0;
      w = dimX -> at (0);
      if (w < 0)
	w *= -1;
      break;
    case(GFX_TYPE_VERTICAL_LINE):
      w = 0;
      h = dimY -> at (0);
      if (h < 0)
	h *= -1;
     break;
    case(GFX_TYPE_LINE):
    case(GFX_TYPE_LINE_AA):
    case(GFX_TYPE_RECTANGLE):
    case(GFX_TYPE_RECTANGLE_FILLED):
      w = dimX -> at (0);
      if (w < 0)
	w *= -1;
      h = dimY -> at (0);
      if (h < 0)
	h *= -1;
     break;
    case(GFX_TYPE_BEZIER_CURVE): //even more approximate w and h due to Bezier
    case(GFX_TYPE_POLYGON):
    case(GFX_TYPE_POLYGON_FILLED):
      //get the min and max of each array
      xIter = dimX -> begin () + 1;
      yIter = dimY -> begin () + 1;

      while (xIter != dimX -> end () && yIter != dimY -> end ())
	{
	  if ((*xIter) > maxX)
	    maxX = (*xIter);

	  if ((*xIter) < minX)
	    minX = (*xIter);

	  if ((*yIter) > maxY)
	    maxY = (*yIter);

	  if ((*yIter) < minY)
	    minY = (*yIter);

	  xIter++;
	  yIter++;
	}

      centerOfMassX = minX * -1;
      centerOfMassY = minY * -1;
      
      w = maxX - minX;
      h = maxY - minY;
      break;
    case(GFX_TYPE_TRIGON):
    case(GFX_TYPE_TRIGON_AA):
    case(GFX_TYPE_TRIGON_FILLED):
      //get the min and max of each array
      xIter = dimX -> begin ();
      yIter = dimY -> begin ();

      while (xIter != dimX -> end () && yIter != dimY -> end ())
	{
	  if ((*xIter) > maxX)
	    maxX = (*xIter);

	  if ((*xIter) < minX)
	    minX = (*xIter);

	  if ((*yIter) > maxY)
	    maxY = (*yIter);

	  if ((*yIter) < minY)
	    minY = (*yIter);

	  xIter++;
	  yIter++;
	}

      centerOfMassX = minX * -1;
      centerOfMassY = minY * -1;
      
      w = maxX - minX;
      h = maxY - minY;
      break;
    default:
      break;
    };
}
