#include "SDLResourceToon.h"
#include "SDLResourceAnimation.h"
#include "SDLResourceEngine.h"

#include "SDL.h"
#include "../fileUtils.h"

#include <vector>
using namespace std;

SDLResourceToon :: SDLResourceToon (SDLResourceEngine * sre)
{
  //assigns stuff
  anims = new vector <SDLResourceAnimation *> (0);

  playStatus = ANIMATION_STATUS_STOP;
  currentFrame = 0;
  previousAnimTime = 0;
  timeInCurrentFrame = 0;
  loopCounter = 0;
  currentAnim = NULL_INDEX;
  sdlRE = sre;
}

SDLResourceToon :: ~SDLResourceToon ()
{
  //clears animations before exiting
  if (anims == NULL)
    return;

  vector <SDLResourceAnimation *> :: iterator the_iterator;
  the_iterator = anims -> begin ();

  while(the_iterator != anims -> end ()) {
    delete (*the_iterator);
    the_iterator ++;
  }

  delete anims;
}

bool SDLResourceToon :: loadToon (const string & index)
{
  //reads file, loads animations from there

  xmlDocPtr doc;
  xmlXPathContextPtr xpathCtx; 

  vector <SDLResourceAnimation *> :: iterator the_iterator;

  //jump to the animation section
  
  if (!loadXMLFile (doc, "resources/ToonManifest.xml"))
    {
      xmlFreeDoc(doc);
      return false;
    }

  xpathCtx = xmlXPathNewContext(doc);
  if(xpathCtx == NULL) {
    writeToErrorLog("Error: unable to create new XPath context",0);
    xmlFreeDoc(doc); 
    return false;
  }

  string xpathStart="/ToonManifest/Toon[@ID='";
  xpathStart+=index;
  xpathStart+="']/Animation";

  xmlXPathObjectPtr xpathObj;
  xpathObj=getElementsUsingXPath (doc, xpathCtx, xpathStart, "", "");

  string idString;
  char searchString [255];
  
  if (xpathObj == NULL)
    {
      xmlXPathFreeContext(xpathCtx);
      return false;
    }

  for (int i=0; i<xpathObj->nodesetval->nodeNr; i++)
    {
      sprintf (searchString, "[%u]", i+1);
      idString=xpathStart+searchString;

      anims -> push_back (new SDLResourceAnimation (sdlRE));
      if (!(anims -> back () -> loadAnim (doc, xpathCtx, idString)))
	{
	  anims -> pop_back ();
	}
    }

  //when you're done with the xml document, free it.
  xmlXPathFreeObject(xpathObj);
  xmlXPathFreeContext(xpathCtx);
  xmlFreeDoc (doc);

  if (anims -> size () <= 0)
    {
      //Failed to load shape. Cleanup and exit.
      writeToErrorLog ("Loading Toon: No animation frames found, not loading toon", 0);
      writeToErrorLog (index.c_str(), 1);
      anims -> pop_back ();
      return false;
    }

  return true;
}

bool SDLResourceToon :: chooseAnim (uint8_t animIndex)
{
  //chooses an animation from the loaded 

  //reject out of bounds indecies here

  if (animIndex >= anims -> size ())
    return false;

  currentAnim = animIndex;
  currentFrame = 0;
  playStatus = ANIMATION_STATUS_STOP;  
  return true;
}

void SDLResourceToon :: draw (int32_t x, int32_t y, bool respectCamera, uint8_t orientation)
{
  //checks the time, finds the frame it should draw then draws it on screen, taking anchor position into account.

  if (anims -> at (currentAnim) -> getFrameIndex (currentFrame) == "NULL_INDEX")
    return;

  //Reading the xAnchor and yAnchor:
  //The xPoints and yPoints represent the center of the image. This is useful if you have a frame where the sides extend but you want to keep it centered. So if you expect the frame to move to the right, the anchor is to the LEFT.
  int32_t centerX, centerY;

  centerX = x - anims -> at (currentAnim) -> getXAnchor (currentFrame);
  centerY = y - anims -> at (currentAnim) -> getYAnchor (currentFrame);

  sdlRE -> drawFrame (anims -> at (currentAnim) -> getFrameIndex (currentFrame), centerX, centerY, respectCamera, orientation);

  //increment time and figure out if you should advance frames.
  //Take the amount of time that changed from the last draw and compare it to the duration of this frame. If it is greater, figure out what the next frame is and skip to it.
  if (playStatus == ANIMATION_STATUS_PLAY)
    {
      timeInCurrentFrame += SDL_GetTicks () - previousAnimTime;

      if (timeInCurrentFrame > anims -> at (currentAnim) -> getTimeForFrame (currentFrame))
	{
	  //go to the next frame
	  setNextFrame (timeInCurrentFrame);
	}
      previousAnimTime = SDL_GetTicks ();
    }
  //if the animation is paused, maintain the timer.
  if (playStatus == ANIMATION_STATUS_PAUSE)
    previousAnimTime = SDL_GetTicks ();
}

void SDLResourceToon :: playAnim ()
{
  //plays the animation, advancing frames if need be
  //Start animation.
  if (playStatus == ANIMATION_STATUS_STOP)
    {
      //reset variables
      loopCounter = anims -> at (currentAnim) -> getNumLoops ();
      timeInCurrentFrame = 0;
      currentFrame = 0;
      previousAnimTime = SDL_GetTicks ();
    }

  //play sound at startup here
  sdlRE -> playSound (anims -> at (currentAnim) -> getSoundAtStartup ());
  playStatus = ANIMATION_STATUS_PLAY;
}

void SDLResourceToon :: pauseAnim ()
{
  //pauses the current animation
  playStatus = ANIMATION_STATUS_PAUSE;
}

void SDLResourceToon :: stopAnim ()
{
  //stops the current animation
  playStatus = ANIMATION_STATUS_STOP;
  
  sdlRE -> playSound (anims -> at (currentAnim) -> getSoundAtStop ());
}

void SDLResourceToon :: setNextFrame (uint32_t timeDiff)
{
  //this gets the next frame based on the frame times.

  uint16_t nextFrame = currentFrame;
  uint16_t nextFrameTime;
  bool done = false;
  uint16_t frameCounter = 0;  

  timeInCurrentFrame -= anims -> at (currentAnim) -> getTimeForFrame (currentFrame);

  while (!done)
    {
      //determine next frame
      nextFrame ++;
      frameCounter ++;
      if (nextFrame >= anims -> at (currentAnim) -> getNumFrames ())
	{
	  //animation looped, use loopCounter to determine next action
	  if (loopCounter != 0)
	    {
	      if (loopCounter > 0)
		loopCounter --;
	      nextFrame = anims -> at (currentAnim) -> getRestartFrame ();
	      //play sound at loop here
	      sdlRE -> playSound (anims -> at (currentAnim) -> getSoundAtLoop ());
	    }
	  else if (loopCounter == 0)
	    {
	      currentFrame = nextFrame - 1;
	      done = true;
	      //play sound at end here
	      if (playStatus != ANIMATION_STATUS_STOP)
		stopAnim ();
	      break;
	    }
	}
      if (frameCounter >= 2 * anims -> at (currentAnim) -> getNumFrames ())
	done = true; //How did you loop around? Infinite loop, superfast animation? I'm going to halt it here to prevent animation trouble.

      nextFrameTime = anims -> at (currentAnim) -> getTimeForFrame (nextFrame);
      if (timeInCurrentFrame > nextFrameTime)
	{
	  timeInCurrentFrame -= nextFrameTime;
	}
      else
	{
	  //OK, you found the next frame
	  currentFrame = nextFrame;
	  done = true;
	}
    }
}
