#include "mathUtils.h"
#include "../graphics/GraphicTransformation.h"

#include <cmath>
#include <iostream>
#include <cstdlib>
using namespace std;

bool initializeAngles ()
{
  //populates all lookup tables with values.

  //OK, cycle from 0 to 256. Convert to radians.
  uint16_t bd;

  //sine
  for (bd = 0; bd < 256; bd++)
    {
      sineAngles[bd]=int32_t(1000 * sin(byteDegreesToRadians(bd)));
    }
  //filling in special values
  sineAngles[0]=0;
  sineAngles[64]=1 * 1000;
  sineAngles[128]=0;
  sineAngles[192]=-1 * 1000;

  //cosine
  for (bd = 0; bd < 256; bd++)
    {
      cosineAngles[bd]=int32_t(1000 * cos(byteDegreesToRadians(bd)));
    }
  //filling in special values
  cosineAngles[0]=1 * 1000;
  cosineAngles[64]=0 * 1000;
  cosineAngles[128]=-1 * 1000;
  cosineAngles[192]=0 * 1000;

  for (bd = 0; bd < 256; bd++)
    {
      tangentAngles[bd]=int32_t(1000 * tan(byteDegreesToRadians(bd)));
    }
  //filling in special values
  tangentAngles[0]=0 * 1000;
  tangentAngles[32]=1 * 1000;
  tangentAngles[64]=0xFFFF;
  tangentAngles[96]=-1 * 1000;
  tangentAngles[128]=0 * 1000;
  tangentAngles[192]=-1 * 0xFFFF;
  tangentAngles[224]=-1 * 1000;

  return true;
}

double byteDegreesToRadians (uint8_t bds)
{
  return double(bds) * NUMBER_PI / 128.0;
}

uint8_t radiansToByteDegrees (double radians)
{
  //note: I'm abusing integer casting here so it takes care of negative numbers for me.

  double temp = radians * 128.0 / NUMBER_PI;
  double frac, integ;

  frac = modf(temp, &integ);

  if (frac > 0.5)
    return uint8_t(temp + 1.0);

  return uint8_t(temp);
}

float byteDegreesToDegrees(uint8_t bt)
{
  return (bt * 360.0 / 256.0);
}

int32_t sine (uint8_t bds)
{
  return sineAngles[bds];
}

int32_t cosine (uint8_t bds)
{
  return cosineAngles[bds];
}

int32_t tangent (uint8_t bds)
{
  return tangentAngles[bds];
}

uint8_t arctangent (int run, int rise, uint8_t defaultAngle)
{
  //calculates the arctangent of the rise/run, in byte degrees (out of 256)
  //This uses cmath's atan2 function and is thus slower.

  //if distance is 0, angle is undefined. Send a default angle to return in case of error.
  if (run == 0 && rise == 0)
    return defaultAngle;

  //So arctan doesn't work if the angle is 64 or 192. Check if X is 0 and send 64/192 based on Y direction.
  if (run == 0 && rise > 0)
    return 64;
  else if (run == 0)
    return 192;

  if (rise == 0 && run > 0)
    return 0;
  else if (rise == 0 && run < 0)
    return 128;

  //OK now do arctan. atan2 figures out what quadrant it is in as well, w00t.
  double temp = atan2 (double(rise), double(run));

  //temp is in radians, convert to byte degrees
  return radiansToByteDegrees(temp);
}

bool pointRectangleCollision (int pointX, int pointY, int leftSide, int rightSide, int topSide, int bottomSide)
{
  //returns true if Point is within Side.
  if (pointX < leftSide)
    return false;

  if (pointX > rightSide)
    return false;

  if (pointY < topSide)
    return false;

  if (pointY > bottomSide)
    return false;

  return true;
}

int intAbs (int a)
{
  if (a < 0)
    a *= -1;

  return a;
}

/*
 *      Square root by abacus algorithm, Martin Guy @ UKC, June 1985.
 *      From a book on programming abaci by Mr C. Woo.
 *      Argument is a positive integer, as is result.
 *
 *      I have formally proved that on exit:
 *                 2               2               2
 *              res  <= x < (res+1)     and     res  + op == x
 *
 *      This is also nine times faster than the library routine (-lm).

http://freaknet.org/martin/tape/gos/misc/personal/msc/sqrt/sqrt.html
 */

//This algorithm is based on Martin Guy's code.
int intSqrt (int x)
{
  if (x <= 0)
    return 0;

  int result=0, operand=x, digitToTest=1 << 30;

  //we want to shrink digitToTest until it's 4 bits less than the argument's bits.
  //If it's any bigger there is NO WAY it can be the square root.
  while (digitToTest > operand)
    digitToTest >>= 2;

  while (digitToTest != 0) //stop when the bit is too small.
    {
      //When searching digit by digit, you want to find e, where (r + e) ^ 2 <= x.
      //In binary, e is two bits, or 00 01 10 11.
      //r^2 + 2re + e^2 <= x. e^2 ranges from 0-1001
      //r^2 is what we're looking for.
      //2re is easily handled with bit shifts.

      if (operand >= result + digitToTest)
        {
          operand = operand - (result + digitToTest);
          result = result + (2 * digitToTest);
        }

      result /= 2; //hmm not sure why this is here.
      digitToTest /= 4; //onto the next digit to test.
    }

  return result;
}

void transformPoint (int16_t & xCoord, int16_t & yCoord, GraphicTransformation & gt)
{
  //reads the GraphicTransformation and applies it to the xCoord and yCoord.

  //copy x,y to resultx, resulty
  int16_t resultX=xCoord, resultY=yCoord, temp;
  int32_t length;
  uint8_t angle;

  //Get the anchor.
  if ((gt.getTransformFlag() & GRAPHIC_TRANSFORMATION_ROTATION) > 0)
    {
      resultX-=gt.getRotationAnchorX();
      resultY-=gt.getRotationAnchorY();
    }

  //If scale is set, multiply x,y to resultx,resulty
  if ((gt.getTransformFlag() & GRAPHIC_TRANSFORMATION_SCALE) > 0)
    {
      resultX *= gt.getScaleWidth();
      resultY *= gt.getScaleHeight();
    }

  //3) If rotate is set
  //a) if rotate is 0, do nothing

  //b) if rotate is 128, multiply resultx by -1
  if ((gt.getTransformFlag() & GRAPHIC_TRANSFORMATION_ROTATION) > 0 && gt.getRotationAngle()==128)
    {
      resultX*=-1;
      resultY*=-1;
    }

  //c) if rotate is 64, map (resultx,resulty) to (resulty,resultx)
  if ((gt.getTransformFlag() & GRAPHIC_TRANSFORMATION_ROTATION) > 0 && gt.getRotationAngle()==64)
    {
      temp=resultX;
      resultX=-1 * resultY;
      resultY=temp;
    }

  //d) if rotate is 192, map (resultx,resulty) to (resulty,-resultx)
  if ((gt.getTransformFlag() & GRAPHIC_TRANSFORMATION_ROTATION) > 0 && gt.getRotationAngle()==192)
    {
      temp=resultX;
      resultX=resultY;
      resultY=-1 * temp;
    }

  //e) if rotate is not any of those
  if ((gt.getTransformFlag() & GRAPHIC_TRANSFORMATION_ROTATION) > 0 && (gt.getRotationAngle()%64!=0))
    {
      //translate to polar coordinates with arctangent & intSqrt
      length=intSqrt(resultX*resultX+resultY*resultY);
      angle=arctangent(resultX,resultY);

      //change angle
      angle+=gt.getRotationAngle();

      //translate back to euclidean coordinates with cosine and sine
      resultX=cosine(angle) * length / 1000;
      resultY=sine(angle) * length / 1000;
    }

  //reversing scaling given from the scale.
  if ((gt.getTransformFlag() & GRAPHIC_TRANSFORMATION_SCALE) > 0)
    {
      resultX /= 16;
      resultY /= 16;
    }

  //reapply the anchor.
  if ((gt.getTransformFlag() & GRAPHIC_TRANSFORMATION_ROTATION) > 0)
    {
      resultX+=gt.getRotationAnchorX();
      resultY+=gt.getRotationAnchorY();
    }

  xCoord=resultX;
  yCoord=resultY;
}

void seedRandomization ()
{
  sge_Randomize();
}

int generateRandomNumber (int max)
{
  return sge_Random (0, max);
}

int generateRandomNumber (int min, int max)
{
  return sge_Random (min, max);
}
